Package: ExCluster
Title: ExCluster robustly detects differentially expressed exons
        between two conditions of RNA-seq data, requiring at least two
        independent biological replicates per condition
Version: 1.29.0
Author: R. Matthew Tanner, William L. Stanford, and Theodore J. Perkins
Date: 2018-10-02
Maintainer: R. Matthew Tanner <rtann038@uottawa.ca>
Description: ExCluster flattens Ensembl and GENCODE GTF files into GFF files,
    which are used to count reads per non-overlapping exon bin from BAM files.
    This read counting is done using the function featureCounts from the package
    Rsubread. Library sizes are normalized across all biological replicates, and
    ExCluster then compares two different conditions to detect signifcantly 
    differentially spliced genes. This process requires at least two independent
    biological repliates per condition, and ExCluster accepts only exactly two
    conditions at a time. ExCluster ultimately produces false discovery rates
    (FDRs) per gene, which are used to detect significance. Exon log2 fold
    change (log2FC) means and variances may be plotted for each significantly 
    differentially spliced gene, which helps scientists develop hypothesis and
    target differential splicing events for RT-qPCR validation in the wet lab.
biocViews: ImmunoOncology, DifferentialSplicing, RNASeq, Software
Imports: stats, methods, grDevices, graphics, utils
Depends: Rsubread, GenomicRanges, rtracklayer, matrixStats, IRanges
License: GPL-3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/ExCluster
git_branch: devel
git_last_commit: 9571f20
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 22:30:37 UTC; biocbuild

export('GFF_convert')
export('processCounts')
export('ExCluster')
export('plotExonlog2FC')
export('GRangesFromExClustResults')

# import packages
import('stats')
import('methods')
import('grDevices')
import('graphics')
import('utils')
import('matrixStats')
importFrom('IRanges','IRanges')
import('Rsubread')
import('GenomicRanges')
importFrom('rtracklayer','import')
importFrom('rtracklayer','export')

plotExonlog2FC <- function(results.Data=NULL, out.Dir=NULL, FDR.cutoff=0.05, plot.Type="bitmap"){

    ### make sure R doesn't add factors -- R creators never should have made this default = TRUE
    options(stringsAsFactors=FALSE)
    # check to make sure results.Data parameter is given as input
    if (is.null(results.Data) == TRUE){
        stop(call="You must assign the results.Data argument a variable containing the results of an ExCluster analysis.
    For example, if previous ExCluster results were assigned to clustResults, you would specify: results.Data=clustResults")
    }
    # check to make sure out.Dir is specified
    if (is.null(out.Dir) == TRUE){
        stop(call="plotExClustResults requires a direcotry to write images to, but was not specified.
    Please specify the full file path to a folder to which exon log2FC plots may be written.")
    }else{
        # now that out.Dir is specified, make sure its parent directory exists
        if (dir.exists(out.Dir) == FALSE){
            # try to make out.Dir no matter what
            try(dir.create(out.Dir, recursive=TRUE, showWarnings = FALSE), silent=TRUE)
        }
    }

    ### now make sure we can write to the out.Dir
    WriteCheck <- file.access(out.Dir, mode=2)
    if (WriteCheck != 0){
        stop(call="ExCluster did not have write priveledges for the out.Dir you specified.
             Please ensure you specify a valid directory that can be created and written to by R.")
    }

    ### now check bitmap and PNG settings for plot.Type
    if (is.null(plot.Type) == TRUE){
        plot.Type <- "bitmap"
    }else if (plot.Type != "bitmap" && plot.Type != "PNG"){
        plot.Type <- "bitmap"
    }

    ### now check that PNG and/or bitmap can be plotted
    # plot.Check is FALSE by default, unless proven TRUE
    plot.Check <- FALSE
    if (plot.Type == "bitmap"){
        # check if bitmap can be plotted
        test.bitmap <- substr(testBMplot(out.Dir)[1],1,5)
        if (test.bitmap == "Error"){
            # check if PNG can be plotted
            test.PNG <- substr(testPNGplot(out.Dir)[1],1,5)
            plot.Type <- "PNG"
            if (test.PNG != "Error"){
                plot.Check <- TRUE
            }
        }else{
            plot.Check <- TRUE
        }
    }else{
        # if plot.Type == "PNG", repeat the steps above in reverse
        test.PNG <- substr(testPNGplot(out.Dir)[1],1,5)
        if (test.PNG == "Error"){
            test.bitmap <- substr(testBMplot(out.Dir)[1],1,5)
            plot.Type <- "bitmap"
            if (test.bitmap != "Error"){
                plot.Check <- TRUE
            }
        }else{
            plot.Check <- TRUE
        }
    }

    ### make sure either bitmap or PNG passed
    if (plot.Check == FALSE){
        stop(call = ExCluster_errors$plot_type_failure)
    }

    ############################# FUNCTIONS ###############################

    ErrorBar <- function(x,y1,y2,MaxXvalue,BinSize){
        ErrorWidth <- min((MaxXvalue*0.015),(BinSize/3))
        lines(c(x,x),c(y1,y2))
        lines(c(x-ErrorWidth,x+ErrorWidth),c(y1,y1))
        lines(c(x-ErrorWidth,x+ErrorWidth),c(y2,y2))
    }

    ########################### NOW PREPARE DATA ###########################

    ### sort results.Data
    results.Data <- results.Data[order(results.Data$EnsG),]

    ### Target genes of interest
    TargetGenes <- unique(results.Data$EnsG[which(results.Data$FDR <= FDR.cutoff)])

    if (length(TargetGenes) > 0){
        ### grab IDs to make data parsing faster
        IDs <- gsub("\\:.*","",results.Data$EnsG)
        IDs <- gsub("\\..*","",IDs)
        IDs <- substr(IDs,(nchar(IDs[1])-9),nchar(IDs[1]))
        # Now add the IDs to results.Data
        results.Data$IDs <- IDs

        ### Do the same for TargetGenes
        IDs <- gsub("\\:.*","",TargetGenes)
        IDs <- gsub("\\..*","",IDs)
        IDs <- substr(IDs,(nchar(IDs[1])-9),nchar(IDs[1]))

        ### loop through results looking for gene data so we can grab FDR, min/max results.Data, etc.
        Start <- 1
        Counter <- 1
        RemovalIndices <- NULL

        Start <- 1

        for (i in 2:nrow(results.Data)){
            # check to see if we have reached a new gene
            if ((results.Data$IDs[i] != results.Data$IDs[(i-1)]) == TRUE | i == nrow(results.Data)){
                # determine stop of gene
                if (i == nrow(results.Data)){
                    Stop <- i
                }else{
                    Stop <- (i-1)
                }
                # now check to see if the current gene of interest is in TargetGenes
                if (results.Data$IDs[Start] == IDs[Counter]){
                    Counter <- min((Counter + 1),length(IDs))
                }else{
                    RemovalIndices <- c(RemovalIndices, c(Start:Stop))
                }
                # reset start for next gene
                Start <- i
            }
        }

        if (length(RemovalIndices) > 0){
            ### Now remove the rows specified in RemovalIndices to yield significant gene data
            sig.Results <- results.Data[-c(RemovalIndices),]
            rm(results.Data)
        }else{
            sig.Results <- results.Data
        }
        # calculate length of each exon bin
        sig.Results$End <- as.numeric(sig.Results$End)
        sig.Results$Start <- as.numeric(sig.Results$Start)
        sig.Results$Length <- sig.Results$End - sig.Results$Start + 1

        ############################# NOW PLOT DATA #############################

        ### loop through signficant results, plotting each gene
        Start <- 1
        for (i in 2:nrow(sig.Results)){
            # check to see if we have reached a new gene
            if ((sig.Results$IDs[i] != sig.Results$IDs[(i-1)]) == TRUE | i == nrow(sig.Results)){
                # determine stop of gene
                if (i == nrow(sig.Results)){
                    Stop <- i
                }else{
                    Stop <- (i-1)
                }
                ### Grab temporary gene data to plot
                TempData <- sig.Results[Start:Stop,]
                # Set up data variables to be used in plotting
                TargetGene <- TempData$EnsG[1]
                TargetName <- TempData$Gene_name[1]
                TargetFDR <- TempData$FDR[1]
                # ceiling of abs(log2FC) + SD + 1 log2FC to set for +/- y-axis limits
                Maxlog2FC <- ceiling(max(abs(TempData$log2FC) + sqrt(TempData$log2Var)))
                # maximum x-axis
                MaxX <- sum(TempData$Length) + nrow(TempData) - 1

                ### now set up png for plotting
                # check to see if we are plotting bitmap or PNG
                if (plot.Type == "bitmap"){
                    bitmap(file=paste(out.Dir,"/",TargetGene,"_",TargetName,".png",sep=""), type="png16m",
                           width=6,height=4,units="in",res = 400, taa=4,gaa=4)
                }
                if (plot.Type == "PNG"){
                    png(filename = paste(out.Dir,"/",TargetGene,"_",TargetName,".png",sep=""),
                        width=6,height=4,units="in",res = 400)
                }
                par(ps = 12, cex = 1, cex.main = 1,mar=(c(5, 5, 4, 2)+0.2))
                plot(c(0,MaxX), c(-(Maxlog2FC),Maxlog2FC) , type= "n",
                     ylab = "log2 fold change (log2FC)",xlab = "nucleotide position (number of base pairs)",
                     main = paste(TargetName,"   ",TargetGene,"    ","FDR=",TargetFDR,sep=""),cex.main=0.9)
                # add a light grey line at log2FC = 0
                lines(x = c(-500,MaxX+500), y = c(0,0), col = "grey")
                # run plotting code
                # Start of Exon = 0
                ExonStart <- 0
                # Bar height
                BH <- Maxlog2FC/60
                for (j in seq(nrow(TempData))){
                    rect(ExonStart, TempData$log2FC[j]-BH, ExonStart + TempData$Length[j],
                         TempData$log2FC[j]+BH, col = "red")
                    ErrorBar((ExonStart + (TempData$Length[j] / 2)),(TempData$log2FC[j]-sqrt(TempData$log2Var[j])),
                             (TempData$log2FC[j]+sqrt(TempData$log2Var[j])),MaxX,TempData$Length[j])
                    ExonStart <- ExonStart + TempData$Length[j] + 1
                }
                dev.off()
                # reset the Start variable for the next gene
                Start <- i
            }
        }
    }
}

processCounts <- function(bam.Files=NULL,sample.Names=NULL,annot.GFF=NULL,GFF.File=NULL, paired.Reads=FALSE,
                          stranded.Reads=FALSE, out.File=NULL, temp.Dir=NULL, num.Cores=NULL){

    ### make sure R doesn't add factors -- R creators never should have made this default = TRUE
    options(stringsAsFactors=FALSE)
    # make sure that bam.Files were entered when running processCounts
    if(is.null(bam.Files) == TRUE){
        stop(call = ExCluster_errors$BAM_argument_missing)
    }
    #now make sure the sample.Names variable is set
    if(is.null(sample.Names) == TRUE){
        stop(call = ExCluster_errors$sample_name_argument_missing)
    }
    # make sure that 'paired.Reads' variable is logical
    if(is.logical(paired.Reads) == FALSE){
        stop(call = ExCluster_errors$paired_reads_not_logical)
    }
    # Check to make sure the number of sample & sample names are the same
    if (length(bam.Files) != length(sample.Names)){
        stop(call = ExCluster_errors$BAM_sample_length_unequal)
    }

    ########################## Read counts with Rsubread ##########################

    ### check to make sure either the GFF data is present, or a GFF file path is given.
    if(is.null(annot.GFF) == TRUE){
        if (is.null(GFF.File) == FALSE){
            if (file.exists(GFF.File) == TRUE){
                annot.GFF <- read.table(file=GFF.File,header=FALSE,stringsAsFactors=FALSE)
            }else{
                stop(call = ExCluster_errors$GFF_file_doesnt_exist)
            }
        }else{
            stop(call = ExCluster_errors$GFF_file_argument_missing)
        }
    }else{
        ### now check to see if annot.GFF is an S4 object (GRanges) & reformat it
        if (substr(typeof(annot.GFF),1,2) == "S4"){
            annot.GFF <- GRangesToGFF(annot.GFF)
        }
    }

    ### check to make sure the GFF file has 9 columns
    if (ncol(annot.GFF) != 9){
        stop(call=ExCluster_errors$GFF_missing_columns)
    }

    ### now check to make sure that column 9 contains 'ID=', 'Name=', and 'Transcripts=' strings
    ID.indices <- length(grep(pattern = "ID=", as.character(annot.GFF[seq(2),9])))
    Name.indices <- length(grep(pattern = "Name=", as.character(annot.GFF[seq(2),9])))
    Transcripts.indices <- length(grep(pattern = "Transcripts=", as.character(annot.GFF[seq(2),9])))
    # if the lengths of these indices do not sum to 6, throw an error
    if (ID.indices+Name.indices+Transcripts.indices != 6){
        stop(call=ExCluster_errors$GFF_missing_GFF3_fields)
    }

    ### now reformat the GFF file if all the checks passed, to our internal format
    annot.GFF <- reformat_GFF3(annot.GFF)

    ### default nCores = 1 if not specified, or incorrectly specified
    nCores <- 1
    # now check if we are using multiple cores
    if (is.null(num.Cores) == FALSE){
        # check to make sure num.Cores is a 'double' type variable
        if (is.double(num.Cores) == TRUE){
            nCores <- num.Cores
        }else{
            warning(call=ExCluster_errors$non_integer_numcores)
            nCores <- 1
        }
    }

    ### default the 'tmpDir' argument for featureCounts to tempdir() if not specified
    tmpDir <- tempdir()
    # now check if the temp.Dir argument was specified in ExClusters input
    if (is.null(temp.Dir) == FALSE){
        # make sure we can write to temp.Dir
        WriteCheck <- file.access(temp.Dir, mode=2)
        if (WriteCheck != 0){
            tmpDir <- temp.Dir
        }else{
            warning(call = ExCluster_errors$temp_dir_inaccessible)
        }
    }

    ### are reads stranded?
    if (stranded.Reads == FALSE){
        stranded.Reads <- 0
    }else{
        stranded.Reads <- 1
    }

    ### Convert GFF annotations to SAF
    SAF.annot <- data.frame(annot.GFF$V2,annot.GFF$V1,annot.GFF$V4,annot.GFF$V5,annot.GFF$V7)
    colnames(SAF.annot) <- c("GeneID","Chr","Start","End","Strand")

    ### Run featureCounts on BAM files
    ### The authors and original license holders of featureCounts and the Rsubread package make no warranty for its performance
    fC <- featureCounts(files = bam.Files, annot.ext = SAF.annot, isGTFAnnotationFile = FALSE, nthreads = nCores, tmpDir=tmpDir,
            requireBothEndsMapped = FALSE, allowMultiOverlap = TRUE, largestOverlap = FALSE, isPairedEnd = paired.Reads,
            strandSpecific = stranded.Reads)

    message('',"Running library size normalization...",'',sep="\n")

    ### clean up SAF.annot (not needed anymore)
    rm(SAF.annot)

    ### change fC to data frame & add colnames
    DataTable <- data.frame(fC$counts)
    colnames(DataTable) <- c(sample.Names)

    ### run ambiguous read removal if stranded.Reads == FALSE
    if (stranded.Reads == 0){
        # determine how many overlaps each exon bin has
        overlap.GRanges <- parseAmbiguousReads(read.Counts = DataTable, annot.GFF=annot.GFF)
        # set ambiguous reads to zero if reads are unstranded
        if (is.null(overlap.GRanges) == FALSE){
            DataTable[overlap.GRanges,] <- 0
        }
    }

    ######################## Now normalize read counts ########################

    # check if we have at least 1000 rows of data to normalize counts to
    if (nrow(DataTable[which(rowMins(as.matrix(DataTable)) >= 8),]) < 1000){
        # warn the user that they didn't have a good amount of data
        # i.e. if less than 1000 elibile features (i.e. exon bins) were expressed
        warning(call = ExCluster_errors$low_exon_bin_count)

        # simple library size normalization
        sampleSums <- colSums2(as.matrix(DataTable))
        # compute baseMean library size (across all conditions)
        sampleBaseMean <- mean(sampleSums)
        # compute sizeFactor differences for each sample vs basemean
        sizeFactors <- sampleSums/sampleBaseMean
        # now adjust DataTable based on these size factors
        adjusted.Counts <- t(t(DataTable)/sizeFactors)

    }else{
        adjusted.Counts <- normalizeLibrarySizes(DataTable)
    }

    ######################## Final processing & output ########################

    ### write output file if necessary
    if (is.null(out.File) == FALSE){
        WriteCheck <- file.access(dirname(out.File), mode=2)
        if (WriteCheck == 0){
            write.table(adjusted.Counts,file=out.File,sep="\t",row.names=TRUE,
                        col.names=TRUE,quote=FALSE)
        }
    }

    ### return counts and finish function
    return(adjusted.Counts)
    message('',"processCounts function has completed.",sep="\n")
}

ExCluster_errors <- list(

    #################################################################################
    ################################## GFF_convert ##################################
    #################################################################################

    GTF.File_missing =
    "You did not  provide the GFF_convert function a GTF input for an argument.
Please give this function a filepath to a GTF file, such as GTF.File=/path/to/file.gtf,
or provide directly GTF annotations within R to the annot.GTF argument.
These annot.GTF annotations may be a standard data frame or an rtracklayer object",

    bad_GTF_filepath =
        "The GTF file path that you specified does not exist.
Please double check your GTF file's full path & file name.
For example, a GTF.File path for mac/linux users might look like: /User/username/path/to/file.gtf",

    check_GTF_columns =
        "Your GTF file did not have 9 tab delimited columns.
If you have edited your GTF file, please ensure the original number of columns and type of columns are retained.
Additionally, if you have edited your GTF file, please ensure that tabs were used to delimit columns.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    GTF_missing_features = "Your GTF file did not have 'gene', 'transcript', and 'exon' features present in column 3.
If you have edited your GTF file, please ensure the original number of columns and type of columns are retained.
Also double check that your GTF file was read into R without row or column names, as the GTF format has none.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    bad_GTF_start_stop = "The start and stop columns (4 & 5) of your GTF file were not properly formatted.
Please ensure your gtf.file was read correctly, without row or column names from the original GTF file.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    start_stop_not_numeric = "There were negative distances between your stop and start locations.
The start/stop locations of your genomic features in this GTF file appear to be corrupted.
Please ensure column 4 of your GTF file is start location, and column 5 is stop location.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    bad_GTF_plus_minus = "The strand column of your GTF file (column 7) had more than 2 unique values.
This column should only have '+' and '-' values. Please ensure your GTF file is read into R without row/column names.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    GTF_plus_minus_missing = "The strand column of your GTF file (column 7) did not contain '+' or '-' values.
This column should only have '+' and/or '-' values. Please ensure your GTF file is read into R without row/column names.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    GTF_missing_gene_id = "Column 9 in your GTF file did not contain gene_id and/or transcript_id values in the correct places.
Please verify that your GTF file was read into R without row or column names, and that each row has a gene_id in column 9.
If this error persists, please re-download your GTF annotations from the source database, and try again.",

    #################################################################################
    ################################# processCounts #################################
    #################################################################################

    BAM_argument_missing = "No BAM files were entered when running processCounts().
Please ensure the bam.Files argument was assigned a character vector of full BAM file paths.
Alternatively, you can assign the BAM file names as a character vector to the bam.Files variable, such as:
    bam.Files <- c('/path/to/file1.bam','/path/to/file2.bam',...)",

    sample_name_argument_missing = "No sample names were entered to correspond to your BAM files.
Please make sure the sample.Names argument is assigned a character vector when calling processCounts().
Alternatively, you can assign the character vector to the sample.Names variable as follows:
sample.Names <- c('control1_rep1','control1_rep2',...)",

    paired_reads_not_logical = "You did not correctly enter the 'pairedReads' variable as TRUE or FALSE.
Please use pairedReads=TRUE or pairedReads=FALSE when calling processCounts()",

    BAM_sample_length_unequal = "The number of BAM files and sample names are not the same!
Please ensure the lengths of bam.Files and sample.Names variables are the same.",

    GFF_file_doesnt_exist = "The GFF file path you provided to GFF.File does not exist.
Please verify you have given the correct, full filepath including file extension.
    For example, a file path may look like: /Users/username/path/to/file.gff",

    GFF_file_argument_missing = "The GFF.File argument was not provided when processCounts was run, nor was the annot.GFF argument assigned.
One of annot.GFF or GFF.File must be specified when running processCounts().
The GFF.File argument should be a full file path to a GFF file, such as: /Users/username/path/to/file.gff
Alternatively, you can give the annot.GFF argument a GFF data frame from the GFF_convert function, such as: annot.GFF = GFF",

    GFF_missing_columns = "Your GFF file did not have 9 columns.
Please re-run GFF_convert from an unaltered GTF file, and pass that output to processCounts.
Consult the vignette if you are unsure how to proceed.",

    GFF_missing_GFF3_fields = "Your GFF file did not have ID, Name, and Transcript fields properly formatted in column 9.
Please ensure you are using a GFF file produced from the ExCluster GFF_convert function.
Unfortunately this function can only accept GFF3 formatted GFF files from GFF_convert.",

    non_integer_numcores = "You entered a non-integer value for num.Cores -- it should have a value of >= 1.
Defaulting number of cores to 1 for this run.",

    temp_dir_inaccessible = "The temp.Dir path you provided was not accessible by R for read/write.
    Please ensure the temp.Dir argument is given a valid folder path with read/write permissions.
    Defaulting temporary directory to R's tempdir() function.",

    low_exon_bin_count = "You had fewer than 1000 exon bins with RNA-seq reads counted into them across conditions.
Please be aware that ExCluster expects whole transcriptome GFF and BAM files counted, not individual chromosome data.
Without these full files, ExCluster cannot properly correct library sizes and tune statistics.
    >> If you are simply testing an ExCluster example, please ignore this warning.",

    #################################################################################
    ################################### ExCluster ###################################
    #################################################################################

    exon_counts_missing = "You did provide a normalized exon count matrix to the exonCounts argument.
Please obtain normalized exon counts from processCounts() and assign the resutls to exonCounts.
    For example, if you followed the vignette, you should have a normCounts variable.
    In this case, you could specify: exonCounts=normCounts",

    cond_nums_missing = "You did not provide condition numbers to the cond.Nums argument.
Please ensure you specify which columns of your exon read counts correspond to which condition.
For example, specify cond.Nums=c(1,1,1,2,2,2) if you have 6 samples between 2 conditions.
Please note the order of the numbers must correspond to the order of the columns.",

    exon_count_length_incorrect =
"Your number of exonCounts columns did not equal the length of your number of condition identifiers.
Please ensure that you entered correct condition identifiers to cond.Nums, such as cond.Nums=c(1,1,1,2,2,2).
Additionally, please also ensure your normalized counts provided to exonCounts have only count data columns.
All Gene:ExonBin identifiers should be provided as rownames. If you are unsure, please refer to the vignette.",

    exon_read_counts_duplicated = "One of your normalized exon count columns was duplicated!
If this is a mistake, please double check your normalized count table, or re-run processCounts().
However, if you were attempting to duplicate columns to analyze data without biological replicates, this will not work.
ExClust requires biological replicates to estimate variance within conditions, and duplicate columns yield variances of zero.",

    GFF_annotations_missing = "You must provide either GFF annotation data or a GFF file path.
Please assign the annot.GFF argument a variable name containing the GFF data, such as annot.GFF=GFF.
Alternatively, please specify a full GFF file path, such as: GFF.File=/Users/username/path/to/file.gff",

    GFF_file_inaccessible = "The GFF file path you entered did not exist.
Please verify that you have assigned the exactly correct path to the GFF.File argument.
For example, your argument when running ExCluster should look something like this: GFF.File=/Users/username/path/to/file.gff",

    improper_cond_nums = "You did not specify exactly 2 condition numbers to the cond.Nums argument.
For example, if your data contains two conditions with three replicates each, please use: cond.Nums=c(1,1,1,2,2,2).
If your count data alternates condition columns, such as cond1, cond2, cond1, cond2, etc., use: cond.Nums=c(1,2,1,2,1,2)",

    not_enough_replicates_cond1 = "You only provided biological replicate for your first condition.
ExCluster, unfortunately, requires at least 2 biological replicates per condition to function.
If this was a mistake, please re-enter your cond.Nums argument with at least 2 replicates per condition number.",

    not_enough_replicates_cond2 = "You only provided biological replicate for your second condition.
ExCluster, unfortunately, requires at least 2 biological replicates per condition to function.
If this was a mistake, please re-enter your cond.Nums argument with at least 2 replicates per condition number.",

    ExClust_GFF_bad_colnum = "You did not enter a valid GFF annotation data frame or a proper path to your GFF file.
It is possible that your GFF file is not properly formatted with the expected 9 columns.
Please re-run GFF_convert and then re-assign either annot.GFF or GFF.File when calling ExCluster.
See the ExCluster manual and vignette for more information.",

    GFF_missing_exon_feature = "Your GFF file did not have column 3 beginning with 'exon' in all cases.
Please ensure your GFF file is produced from the GFF_convert function, and read into R without row/column names.
If this error persists, please re-produce your GFF file from a GTF file using GFF_convert.",

    GFF_numeric_error = "The GFF file start and end genomic coordinate columns (4 & 5) were not properly formatted numbers.
Please ensure your GFF annotations were produced from the GFF_convert function.
If you are reading saved GFF annotations from a file, please ensure no row or column names were applied to the file.
If this error persists, please re-produce your GFF file from a GTF file using GFF_convert.",

    GFF_negative_dists = "There were negative distances between your stop and start locations in your GFF file.
Please ensure your GFF annotations were produced from the GFF_convert function.
If you are reading saved GFF annotations from a file, please ensure no row or column names were applied to the file.
Columns 4 & 5 of the GFF file format should contain start/stop locations with only numeric values throughout.
If this error persists, please re-produce your GFF file from a GTF file using GFF_convert.",

    bad_GFF_strand_column = "The strand column of your GFF file (column 7) had more than 2 unique values.
This column should only have '+' and '-' values. Please ensure your GFF file is read into R without row/column names.
If this error persists, please re-produce your GFF file from a GTF file using GFF_convert.",

    GFF_plus_minus_missing = "The strand column of your GTF file (column 7) did not contain '+' or '-' values.
This column should only have '+' and '-' values. Please ensure your GFF file is read into R without row/column names.
If this error persists, please re-produce your GFF file from a GTF file using GFF_convert.",

    match_GFF_counts_failed = "The GFF file you provided does not match the GFF file used to count exonic reads.
Please consider converting your GTF file to GFF again, and then re-counting exon reads.
Otherwise, please make sure the same GFF file used to count exon reads is provided to ExCluster's input arguments.",

    too_few_pvalues = "You have entered transcriptome data which contains fewer than 5000 expressed genes.
ExCluster cannot run advanced statistics (yielding better tuned FDRs) with < 5000 expressed genes.
Please avoid running ExCluster on sub-sections of a genome (i.e. only certain chromosomes),
as background expressed genes are used to tune statistics.

If you are running test data to get ExCluster working, ignore this warning.",

    ExClust_bad_outDir = "You specified an out.Dir directory for ExCluster that could not be written to.
    Please provide a valid directory that ExCluster can write files to.
    ExCluster will finish running, however your file has not been written.",

    FDR_cutoff_too_high = "FDR.cutoff was assigned higher than 0.2, which is not allowed.
    Using FDR cutoffs above 20% generates unnecessarily high false discovery rates.
    Defaulting to 0.05 FDR for this run of ExCluster().",

    plot_results_no_outDir = paste("You specified that plot.Results=TRUE, ",
                "but out.Dir was not specified or could not be written to. ",
                "out.Dir must be assigned a valid directory that can be created and/or written to. ",
                "Your exon log2FCs have not been plotted, however, you may re-plot them ",
                "with the plotExClustResults function -- this requires saved ExClust results. ",
                "Please consult the vignette for more instructions.", sep=""),

    plot_type_failure = paste("ExCluster was unable to plot either PNG or bitmap. ",
                              "This error commonly occurs because your operating system ",
                              "lacks both Ghostscript and X11 forwarding. Please ensure ",
                              "R has access to either Ghostscrpit or X11 forwarding.",sep="")

)

GRangesFromExClustResults <- function(results.ExClust=NULL){

    ############################################################################################
    ############################## Very ExClust Results integrity ##############################
    ############################################################################################

    ### make sure column names are correct (all columns up until read count data)
    expected.Names <- c("EnsG","Exon_bin","Gene_name","Cluster","log2FC","log2Var","pval","FDR",
                        "Chr","Strand","Start","End")

    ### make sure that results.ExClust is a data frame with at least 2+ columns
    if (length(ncol(results.ExClust)) != 1){
        if(ncol(results.ExClust) < 2){
            stop(call="You did not enter a valid data structure for results.ExClust with columns.
Please ensure you pass results.ExClust a data structure from the main ExCluster function.
This data structure should have 12 columns at minimum, plus more if read count columns are present.")
        }
    }

    ### make sure that results.ExClust is a data frame with at least 2+ rows
    if (length(nrow(results.ExClust)) != 1){
        if(nrow(results.ExClust) < 2){
            stop(call="You did not enter a valid data structure for results.ExClust with more than 1 row.
Please ensure you pass results.ExClust a data structure from the main ExCluster function.
This data structure should have thousands of rows (exon bins) for any organism transcriptome.")
        }
    }

    ### now check to see if these expected names match actual names
    equal.Vectors <- all.equal(colnames(results.ExClust)[seq(length(expected.Names))], expected.Names)
    # if these vectors are not equal, throw an error
    if (equal.Vectors == FALSE){
        stop(call="The first 12 column names of results.ExClust did not match the expected format.
Please ensure you are passing the ExCluster main function results table directly to results.ExClust.")
    }

    ### check to make sure the Start/End columns are numeric with positive differences only
    # test to see if the math will work (will not work if columns are not numeric)
    coord.Diff <- try(as.numeric(results.ExClust$End) - as.numeric(results.ExClust$Start),silent = TRUE)
    # check to see if we have an error
    if (substr(coord.Diff[1],1,5) == "Error"){
        stop(call = "The Start/End columns of your ExClust results table were not properly formatted.
These columns should consist of only numeric values, but instead contained non-numeric values.
Please ensure you are passing the ExCluster main function results table directly to results.ExClust.")
        }
    # if that did not fail, the code continues and we verify that all stop - start subtractions are >= 0
    if (min(coord.Diff) < 0){
        stop(call = "There were negative distances between your Start/End locations in your ExClust results.
These distances should always be positive when subtracting Start locations from End locations, per exon bin.
Please ensure you are passing the ExCluster main function results table directly to results.ExClust.")
    }

    ### check to make sure the Strand column unique values have at least "+" or "-"
    # unique values of Strand column in the ExClust results
    strand.Values <- unique(results.ExClust$Strand)
    # is "+" present? if true this will == 1, if false this will == 0
    positive.Length <- length(which('+'%in%strand.Values))
    # is "-" present? if true this will == 1, if false this will == 0
    negative.Length <- length(which('-'%in%strand.Values))
    # make sure there are no more than 2 unique values contained within the Strand column
    if (length(strand.Values) > 2){
        stop(call = "The Strand column of your ExClust results contained more than 2 unique values.
This column should only have '+' and '-' values.
Please ensure you are passing the ExCluster main function results table directly to results.ExClust.")
    }
    # if the previous check passed, make sure at least one of '-' or '+' is present
    if ((positive.Length + negative.Length) < 1){
        stop(call = "The Strand column of your ExClust results did not contain '+' or '-' values.
This column should only have '+' and '-' values.
Please ensure you are passing the ExCluster main function results table directly to results.ExClust.")
    }

    #############################################################################################
    ########################## Now convert ExClust Results to GRanges ###########################
    #############################################################################################

    # convert to GRanges
    ExClustResults.GRanges <- GRanges(seqnames=results.ExClust$Chr,
                                      ranges=IRanges(as.numeric(results.ExClust$Start),as.numeric(results.ExClust$End)),
                                      EnsG=results.ExClust$EnsG, Exon_bin=results.ExClust$Exon_bin,
                                      Gene_name=results.ExClust$Gene_name, Cluster=results.ExClust$Cluster,
                                      log2FC=results.ExClust$log2FC, log2Var=results.ExClust$log2Var,
                                      pval=results.ExClust$pval, FDR=results.ExClust$FDR,
                                      counts=results.ExClust[,seq(13,ncol(results.ExClust))])

    # export this GRanges object & end function
    return(ExClustResults.GRanges)
}

### R code from vignette source 'ExCluster.Rnw'

###################################################
### code chunk number 1: ExCluster.Rnw:186-191
###################################################
library(ExCluster)
# load the sub-sampled GTF file path from the ExCluster package
GTF_file <- system.file("extdata","sub_gen.v23.gtf", package = "ExCluster")
# now run GTF\_file without assigning a GFF\_file to write out, assigning the results to the GFF object
GFF <- GFF_convert(GTF.File=GTF_file)


###################################################
### code chunk number 2: ExCluster.Rnw:198-206
###################################################
# specify the path to the ExCluster package
ExClust_Path <- system.file(package="ExCluster")
# now find the bam files within that folder
bamFiles <- list.files(ExClust_Path,recursive=TRUE,pattern="*.bam",full.names=TRUE)
# assign sample names (only 2 replicates per condition in this example)
sampleNames <- c("iPSC_cond1_rep1","iPSC_cond1_rep2","iPSC_cond2_rep1","iPSC_cond2_rep2")
# now run processCounts, with paired.Reads=TRUE because we are counting paired-end data
normCounts <- processCounts(bam.Files=bamFiles, sample.Names=sampleNames, annot.GFF=GFF, paired.Reads=TRUE)


###################################################
### code chunk number 3: ExCluster.Rnw:213-219
###################################################
# assign condition numbers to your samples (we have 4 samples, 2 replicates per condition)
condNums <- c(1,1,2,2)
# now we run ExCluster, assigning its output to the ExClustResults variable
# we are not writing out the ExClustResults table, nor are we plotting exons
# we also use combine.Exons=FALSE to discover one 'significant' gene for example plot purposes
ExClust_Results <- ExCluster(exon.Counts=normCounts,cond.Nums=condNums,annot.GFF=GFF, combine.Exons=FALSE)


###################################################
### code chunk number 4: ExCluster.Rnw:226-231
###################################################
# now we must specify a directory to write images to
# here we use tempdir, but you may substitute another folder path if you wish
outDir <- paste(tempdir(),"/Images/",sep="")
# now we can run our exon log2FC plotting function
plotExonlog2FC(results.Data=ExClust_Results, out.Dir=outDir, plot.Type="PNG")


###################################################
### code chunk number 5: ExCluster.Rnw:243-244
###################################################
GRanges.ExClustResults <- GRangesFromExClustResults(results.ExClust=ExClust_Results)

############################################################################
################################ GFF_convert ###############################
############################################################################

### take rtracklayer objects & reformat them into standard GTF
reformat_GTF <- function(rtracklayer.GTF=NULL){

    ### This function converts rtracklayer imported GTF files into flattened GFF files
    ### it functions by converting rtracklayer format back to GTF, and then calls GFF_convert()

    ### export rtracklayer object to GTF file (temporary gzip file)
    export(rtracklayer.GTF, format = "GTF", con=paste(tempdir(),"/temp.gtf",sep=""))

    ### now read this file back in
    annot.GTF <- read.table(file=paste(tempdir(),"/temp.gtf",sep=""),sep="\t", stringsAsFactors=FALSE)

    ### clean up
    file.remove(paste(tempdir(),"/temp.gtf",sep=""))

    ### return result & end function
    return(annot.GTF)
}

# This function takes the input of GTF data in the specific format produced in the GFF_collapse() function
# Normally this would be GTF data for a single gene
CollapseExons <- function(GR.data){
    # number of rows in the GTF dataframe to be made
    NRows <- nrow(GR.data)
    if (NRows > 1){
        # generate GRanges for GTF
        GR.gtf <- GRanges(seqnames=GR.data$V1, ranges=IRanges(GR.data$V4,GR.data$V5),tx_id=GR.data$tx.id)
        # collapse GRanges for non-overlapping exon bins for GFF (loses metadata)
        GR.gff <- disjoin(GR.gtf)
        # assuming NRows was > 1, now recount the new NRows in the GFF file
        NRows <- length(GR.gff)
        # now map indices for which rows of GR.gff match which rows in GR.gtf
        indices.gtf <- as(findOverlaps(GR.gff, GR.gtf), "List")
        GFF.transcripts <- NULL
        for (n in seq(NRows)){
            GFF.transcripts[n] <- paste(GR.gtf$tx_id[indices.gtf[[n]]],collapse=',')
        }
        GFF.start <- start(GR.gff)
        GFF.end <- end(GR.gff)
    }else{
        GFF.transcripts <- GR.data$tx.id[1]
        GFF.start <- GR.data$V4[1]
        GFF.end <- GR.data$V5[1]
    }

    # now make the rest of the columns for data frame output of GFF data
    GFF.chr <- rep(GR.data$V1[1],NRows)
    GFF.geneid <- rep(GR.data$gene.id[1],NRows)
    GFF.names <- rep(GR.data$gene.name[1],NRows)

    ### generate the data frame
    GFF.dataframe <- cbind(GFF.chr,GFF.geneid,GFF.start,GFF.end,GFF.transcripts,GFF.names)
    # return the data
    return(GFF.dataframe)
}

### this function indexes the start/stop rows of each gene in the GTF file, to speed up processing
IndexGeneStartStop <- function(x){
    Counter <- 1
    gene.names <- NULL
    gene.starts <- NULL
    gene.stops <- NULL
    gene.starts[1] <- 1
    MAXrow <- length(x)
    for (i in seq(length(x))){
        # make a variable for the next gene, but make sure it isn't higher than the max 'i'
        nextValue <- min((i+1),MAXrow)
        if (x[i] != x[(i+1)] || i == length(x)){
            # annotate stop and fill in gene has table info
            gene.stops[Counter] <- i
            gene.names[Counter] <- x[i]
            # now determine if we continue (if we are not at the end of the GTF file)
            if (i != length(x)){
                Counter <- Counter + 1
                gene.starts[Counter] <- i+1
            }
        }
    }
    gene.data <- data.frame(gene.names,gene.starts,gene.stops)
    return(gene.data)
}

### This function transforms GFF annotation data tables to GRanges format
# We do not use rtracklayer because our GFF file contains 10 columns and may not be handled properly
GRangesFromGFF <- function(annot.GFF=NULL){
    # write out temporary GFF file
    write.table(annot.GFF,file=paste(tempdir(),"/temp.gtf3",sep=""),sep="\t",row.names=FALSE,col.names=FALSE,quote=FALSE)
    # now import this file as a GFF3 file with rtracklayer
    GFF.GRanges <- rtracklayer::import(con=paste(tempdir(),"/temp.gtf3",sep=""),format = "GFF3")
    # clean up
    file.remove(paste(tempdir(),"/temp.gtf3",sep=""))
    # export this GRanges object & end function
    return(GFF.GRanges)
}

############################################################################
############################## process_Counts ##############################
############################################################################

### take rtracklayer objects & reformat them into standard GTF
GRangesToGFF <- function(GFF.GRanges=NULL){

    ### This function converts rtracklayer imported GTF files into flattened GFF files
    ### it functions by converting rtracklayer format back to GTF, and then calls GFF_convert()

    ### export rtracklayer object to GTF file (temporary gzip file)
    rtracklayer::export(GFF.GRanges, format = "GFF3", con=paste(tempdir(),"/temp.gff3",sep=""))

    ### now read this file back in
    annot.GFF <- read.table(file=paste(tempdir(),"/temp.gff3",sep=""),sep="\t", stringsAsFactors=FALSE)

    ### clean up
    file.remove(paste(tempdir(),"/temp.gff3",sep=""))

    ### return result & end function
    return(annot.GFF)
}

### reformat the GFF3 format to the format better suited for analysis
reformat_GFF3 <- function(annot.GFF=NULL){
    # make column 2 gene_id:exon_bin
    annot.GFF[,2] <- gsub(".*ID=(.*?);.*", "\\1", annot.GFF[,9])
    # make column 3 into name column
    annot.GFF[,3] <- gsub(".*Name=(.*?);.*", "\\1", annot.GFF[,9])
    # lastly make column 9 into transcripts column
    annot.GFF[,9] <- gsub(".*Transcripts=(.*?)", "\\1", annot.GFF[,9])
    # return & end function
    return(annot.GFF)
}

parseAmbiguousReads <- function(read.Counts=NULL, annot.GFF=NULL){
    ### parse improper use of function
    if (is.null(annot.GFF) == TRUE){
        stop(call="You are attempting to run parseAmiguousReads outside of processCounts.
This function is designed to only be called internally by processCounts.
If you are running processCounts and still receiving this message, try re-generating your GFF file.")
    }
    # means of each row of read counts
    exon.Bin.Means <- rowMeans2(as.matrix(read.Counts))
    # list of exon bin indices with > 5 read counts
    expressed.Exon.Indices <- which(exon.Bin.Means > 1)
    # grab the names of these exon bins
    expressed.Exon.Bins <- rownames(read.Counts)[expressed.Exon.Indices]
    # reduce GFF data based on these exon bin indices
    temp.GFF <- annot.GFF[expressed.Exon.Indices,]
    # generate GRanges
    adjcount.GRanges <- GRanges(seqnames=temp.GFF$V1, ranges=IRanges(as.numeric(temp.GFF$V4),
                                                                      as.numeric(temp.GFF$V5)))
    rm(temp.GFF)
    # count overlaps per feature
    overlap.GRanges <- countOverlaps(adjcount.GRanges, adjcount.GRanges)
    # now determine the number of exon bins with > 1 overlap (1 = with itself)
    overlap.Bins <- which(overlap.GRanges > 1)

    ### now check do we have overlaps?
    if (length(overlap.Bins) > 0){
        # map these overlap.GRanges back onto original data
        original.GRanges <- which(rownames(read.Counts)%in%expressed.Exon.Bins[overlap.Bins])
    }else{
        original.GRanges <- NULL
    }
    # return
    return(original.GRanges)
}

### normalize library size function (data must be normal space, not log2)
# any feature IDs (gene names, exon bins, etc.) must be in rownames, NOT data column
normalizeLibrarySizes <- function(raw.Data=NULL){
    # prevent factors
    options(stringsAsFactors=FALSE)
    # duplicate raw data original data
    original.Data <- raw.Data
    # add 1 count and take the log2 of reads
    raw.Data <- raw.Data+1
    raw.Data <- log2(raw.Data)
    # count columns and rows
    NCols <- ncol(raw.Data)
    NRows <- nrow(raw.Data)
    # remove genes/exons which have fewer than 8 reads (3 in log2 space) in any one condition
    raw.Data <- raw.Data[which(rowMins(as.matrix(raw.Data)) >= 3),]
    # generate base mean
    raw.Data$baseMean <- rowMeans2(as.matrix(raw.Data))
    # sizeFactors will be filled with % to adjust each column to baseMean
    mean.Diff=NULL
    # loop through conditions
    for (i in seq(NCols)){
        # difference between column expression & baseMean
        expr.Diff <- raw.Data[,i] - raw.Data$baseMean
        # now make a matrix to compute Lower & Upper percentages for shorth window
        Quant <- matrix(0,nrow=61,ncol=3)
        for (n in seq(61)){
            # Lower percentile of shorth window
            Lower <- 0.095 + n*0.005
            # Upper percentile of shorth window
            Upper <- Lower+0.5
            # Now compute quantile values for these percentile values & add them to Quant
            Quant[n,seq(2)] <- quantile(expr.Diff, c(Lower, Upper))
            # Compute the size of the shorth quantile window
            Quant[n,3] <- Quant[n,2] - Quant[n,1]
        }
        # Find the smallest shorth quantile window index
        minShorthIndex <- match(min(Quant[,3]),Quant[,3])
        # Now grab the shorth quantile data from that index
        Shorth <- Quant[minShorthIndex,c(1,2)]
        # Calculate the mean difference in log2 space between this raw.Data column & baseMean
        mean.Diff[i] <- 2^(mean(expr.Diff[which(expr.Diff >= Shorth[1] & expr.Diff <= Shorth[2])]))
    }
    ## now adjust raw.Data based on the sizeFactors we have generated
    adjusted.Data <- t(t(original.Data)/mean.Diff)
    # add row and column names
    rownames(adjusted.Data) <- rownames(original.Data)
    colnames(adjusted.Data) <- colnames(original.Data)[seq(NCols)]
    # now return output
    return(adjusted.Data)
}




#####################################################################################################
#######################################  ExCluster Functions  #######################################
#####################################################################################################

### function to parse numbers from Ensembl IDs
parseGeneIDs <- function(gene.Annot=NULL){
    gene.Annot <- gsub("\\:.*","", gene.Annot)
    gene.Annot <- gsub("\\..*","", gene.Annot)
    gene.Annot <- substr(gene.Annot,(nchar(gene.Annot[1]) - 9),nchar(gene.Annot[1]))
    return(gene.Annot)
}

#### Geometric mean function
gm_mean = function(x, na.rm=TRUE){
    exp(sum(log(x[x > 0]), na.rm=na.rm) / length(x))
}

#Compute effect size
Calc.effect.size <- function(Mean1,Mean2,Var1,Var2){
    Effect_size <- (Mean1-Mean2)/(sqrt((Var1+Var2)/2))
}

### function to determine number of clusters
Determine_HC_Distance <- function(x, HC_cutree){
    which(HC_cutree%in%x)
}

### using for loops, saving 1/2 time
Generate_ES_dists <- function(Mean1=NULL,Var1=NULL){
    x=length(Mean1)
    ES_mat <- matrix(0,nrow=x,ncol=x)

    # fill in nth row and jth column data for ES difference matrix
    for (n in seq(x-1)){
        for (j in seq((n+1),x)){
            ES_mat[n,j] <- Calc.effect.size(Mean1[n],Mean1[j],Var1[n],Var1[j])
            ES_mat[j,n] <- ES_mat[n,j] * -1
        }
    }
    return(ES_mat)
}

### function to compute the distances between all  clusters (apply max to its output)
Compare_HC_Distances <- function(NumClusters=NULL, ES_Dists=NULL, HC_indices=NULL){
    ES_Clust_Dist <- max(vapply(seq(NumClusters), FUN=function(z){
        obsDist <- mean(ES_Dists[HC_indices[[z]],-c(HC_indices[[z]])])
        return(obsDist)},FUN.VALUE = c(Res=0)))
    return(ES_Clust_Dist)
}


Permuted_ES_nullhypo <- function(r, Sim_log2FC=NULL, Sim_log2var=NULL, NumRows=NULL,
                                 DISTANCE=NULL, LINKAGE=NULL, NumClusters=NULL){

    pMean1 <- as.numeric(Sim_log2FC[r,])
    pVar1 <- as.numeric(Sim_log2var[r,])

    # ES difference matrix
    pES_permuted <- Generate_ES_dists(pMean1,pVar1)
    # ES distance matrix (Euclidean by default)
    pES_Dists <- as.matrix(dist(pES_permuted,method=DISTANCE))

    if (NumRows > 3){
        ##Generate hclust object
        HC <- hclust(as.dist(pES_Dists),method=LINKAGE)
        ##cut tree
        HC_cutree <- cutree(HC,k=NumClusters)
        ## find row indices per cluster
        HC_indices <- lapply(seq(NumClusters),FUN=function(y){which(HC_cutree==y)})
    }else{
        HC_indices <- seq(NumRows)
    }

    ### generate distance matrix from distance object
    pES_Clust_Dist <- max(vapply(seq(NumClusters), FUN=function(z){
        pDist <- mean(pES_Dists[HC_indices[[z]],-c(HC_indices[[z]])])
        return(pDist)},FUN.VALUE = c(Res=0)))
    return(pES_Clust_Dist)
}

### Note this function was adapted in full or in part from the NbClust R packaage
# NbClust offers no warranty on this function as per their GPL-2 license
centers <- function(cl, x) {
    x <- as.matrix(x)
    n <- length(cl)
    k <- max(cl)
    centers <- matrix(nrow = k, ncol = ncol(x))
    {
        for (i in seq(k)) {
            for (j in seq(ncol(x))) {
                centers[i, j] <- mean(x[cl == i, j])
            }
        }
    }
    return(centers)
}

### Note this function was adapted in full or in part from the NbClust R packaage
# NbClust offers no warranty on this function as per their GPL-2 license
### SD index scattering computation
SD.Average.scattering <- function(cl, x) {
    x <- as.matrix(x)
    n <- length(cl)
    k <- max(cl)
    centers.matrix <- centers(cl, x)
    cluster.size <- numeric(0)
    variance.clusters <- matrix(0, ncol = ncol(x), nrow = k)
    var <- matrix(0, ncol = ncol(x), nrow = k)
    for (u in seq(k)) cluster.size[u] <- sum(cl == u)
    for (u in seq(k)) {
        for (j in seq(ncol(x))) {
            for (i in seq(n)) {
                if (cl[i] == u)
                    variance.clusters[u, j] <- variance.clusters[u, j] + (x[i, j] - centers.matrix[u, j])^2
            }
        }
    }
    for (u in seq(k)) {
        for (j in seq(ncol(x))) variance.clusters[u, j] = variance.clusters[u, j]/cluster.size[u]
    }
    variance.matrix <- numeric(0)
    for (j in seq(ncol(x))) variance.matrix[j] = var(x[, j]) * (n - 1)/n
    Somme.variance.clusters <- 0
    for (u in seq(k)) Somme.variance.clusters <- Somme.variance.clusters +
        sqrt((variance.clusters[u, ] %*% (variance.clusters[u, ])))
    stdev <- (1/k) * sqrt(Somme.variance.clusters)
    scat <- (1/k) * (Somme.variance.clusters/sqrt(variance.matrix %*% variance.matrix))
    scat <- list(stdev = stdev, centers = centers.matrix, variance.intraclusters = variance.clusters, scatt = scat)
    return(scat)
}

### Note this function was adapted in full or in part from the NbClust R packaage
# NbClust offers no warranty on this function as per their GPL-2 license
#### SD Index distance computation
SD.Distance <- function(cl,x){
    Clust_Dists <- array()
    Dist_Counter <- 1
    MAX_cl <- max(cl)
    for (m in seq(MAX_cl)){
        N <- seq(MAX_cl)
        N <- N[-c(m)]
        for (n in N){
            Clust_Dists[Dist_Counter] <- mean(unlist(x[which(cl%in%m),which(cl%in%n)]))
            Dist_Counter <- Dist_Counter + 1
        }
    }
    Mean_Dist <- as.numeric(mean(Clust_Dists))
    return(Mean_Dist)
}

### Note this function was adapted in full or in part from the NbClust R packaage
# NbClust offers no warranty on this function as per their GPL-2 license
#### SD index final computation, combining scattering + distance results
SD.Index2 <- function(x, clmin, cl) {
    x <- as.matrix(x)
    Scatt <- SD.Average.scattering(cl, x)$scatt
    Dis0 <- SD.Distance(cl, x)
    Alpha <- SD.Distance(clmin, x)
    SD.indices2 <- Dis0/Alpha + (1-Scatt)
    return(SD.indices2)
}

### Note this function was adapted in full or in part from the NbClust R packaage
# NbClust offers no warranty on this function as per their GPL-2 license
Cutree_SD.Index <- function(DISTANCE=NULL, NRows=NULL, HC=NULL, ES_mat=NULL){
    ## hard coded at minimum of 2 clusters
    SD_values <- array()
    for(C in seq(2,NRows)){
        ## cut tree based on observed number of clusters C
        cl1 <- cutree(HC,k=C)
        ## cut tree into the minimum number of clusters (k=2)
        clmin <- cutree(HC, k=2)
        x <- as.matrix(dist(ES_mat,method = DISTANCE))
        SD_values[(C-1)] <- as.numeric(SD.Index2(x,clmin,cl1))
        if (SD_values[(C-1)] < max(SD_values)){
            break
        }
    }
    ### estimated number clusters is the maximized SD_value, + 1 (because we start at 2 clusters)
    Estimated_Clustnum <- (C-1)
    res <- cutree(HC,k=Estimated_Clustnum)
    return(res)
}

### function for estimating the lowest null hypothesis p-value (very rough estimate)
# x = p-value vector
EstNullHypo <- function(x){
    # number of p-values in every interval of 0.02
    NumEstNull <- array()
    for (i in seq(28)){
        Lower <- (i)/400
        Upper <- (i+7)/400
        NumEstNull[i+1] <- length(x[which(x >= Lower & x < Upper)])
    }
    # check to see if p-values have hit a low plateau (null hypothesis)
    for (i in seq(20)){
        # Check maximum number of pvals in the 4 below the current i
        NumBelow <- NumEstNull[seq((i+1),(i+5))]
        # Convert these to TRUE/FALSE
        NumBelowBoolean <- NumEstNull[i] > NumBelow
        # Now see how many of these bins the current NumEstNull[i] is less than
        LessThan <- length(which(NumBelowBoolean == FALSE))
        if (LessThan >= 2) break
    }
    NumEstNull <- min(i/400,0.05)
    # now your i value divided by 400 is your pvalue cutoff
    PValCutoff <-i/400
    return(PValCutoff)
}

## function for correcting FDR -- to be run with lapply()
# x = the p-value vector
# y = the number of estimated True p-values
# z = the number of estimated Null Hypothesis pvalues
FDRcalc <- function(x,y,z){
    # set p FDR values array to be filled
    FDRvalues <- NULL
    # compute the prior odds ratio (NumTrue/NumNull)
    PriorOdds <- y/z

    # now loop through pvalues
    for (n in seq(length(x))){
        # check to make sure that we aren't on the first value, and the previous value isn't FDR == 1
        if (n > 1 && FDRvalues[(n-1)] >= 0.99) {
            FDRvalues[n] <- 1
        }else{
            ### now generate a null hypothesis 2 times to estimate the NumExp and Num Obs
            NumExp <- NULL
            NumObs <- NULL
            for (h in seq(3)){
                # generate pure null hypothesis distribution
                NullHypo <- sort(runif(z,0,1))
                # number of expected p-values based on the null hypothesis distribution
                NumExp[h] <- floor(length(which(NullHypo <= x[n])))
                # number of observed TRUE p-values less than or equal to x[n]
                NumObs[h] <- length(which(x <= x[n])) - NumExp[h]
                # make sure NumObs is not negative
                NumObs[h] <- max(NumObs[h],0)
            }
            # now take the average of these estimates
            NumExp <- NumExp[1]
            NumObs <- NumObs[1]

            # compute the likelihood ratio
            L_Ratio <- NumObs/NumExp
            # compute posterior odds ratio, which is the PriorOdds * the likelihood ratio
            PostOdds <- PriorOdds * L_Ratio
            # compute posterior probability
            PostProb <- 1/(1+(1/PostOdds))
            # now compute FDR value, which is 1 minus the posterior probability
            FDRvalues[n] <- 1 - PostProb
        }
    }

    ### now loop back through FDRs and make sure that no FDR is lower than higher ranked values, stop when FDRs==1
    # find the limit at which FDR==1
    FDRlim <- min(which(FDRvalues == 1),(length(x)-2))
    # loop through each FDR value
    for (n in seq(2,FDRlim[1])){
        # make sure equal p-values have the same FDR for pvals below
        if (x[n] == x[(n-1)]){
            if (FDRvalues[(n-1)] > FDRvalues[n]){
                FDRvalues[(n-1)] <- FDRvalues[n]
            }
        }
        # make sure equal p-values have the same FDR for pvals above
        if (x[n] == x[(n+1)]){
            if (FDRvalues[n] > FDRvalues[(n+1)]){
                FDRvalues[n] <- FDRvalues[(n+1)]
            }
        }

        # compute the minimum FDR above FDRvalues[n]
        MaxN <- min((n+500),FDRlim[1])
        MinFDR <- min(FDRvalues[seq((n-1),MaxN)])
        # now replace the FDRvalues[n] if FDRvalues[n] > Min FDR
        if (FDRvalues[(n-1)] > MinFDR){
            FDRvalues[(n-1)] <- MinFDR
        }
    }
    return(FDRvalues)
}

### function to check if we can plot a PNG
testPNGplot <- function(test.Dir=NULL){
    test.PNG <- try(
        {  png(filename=paste(test.Dir,"/test.png",sep=""))
            plot(c(1,2,3),c(1,2,3))
            dev.off()
            file.remove(paste(test.Dir,"/test.png",sep=""))}
        , silent = TRUE)
    return(test.PNG)
}

### function to check if we can plot a PNG
testBMplot <- function(test.Dir=NULL){
    test.BM <- try(
        {  bitmap(file=paste(test.Dir,"/test.png",sep=""))
            plot(c(1,2,3),c(1,2,3))
            dev.off()
            file.remove(paste(test.Dir,"/test.png",sep=""))}
        , silent = TRUE)
    return(test.BM)
}

ExClust_main_function <- function(newlog2FC=NULL, Indices1=NULL, Indices2=NULL, NumReps1=NULL, NumReps2=NULL){

    ################################  SETUP PARAMETERS & FUNCTIONS ################################

    ## clustering method = euclidean
    DISTANCE <- "euclidean"

    ## linkage == complete
    LINKAGE <- "complete"

    ### Function for running null hypothesis non-parametric iterations
    generate_Nullhypo_Dists <- function(NumIterations=NULL,gene_FC=NULL,Gene_R1=NULL,Gene_R2=NULL, NumRows=NULL){
        ## rows = genes, columns = exons
        Sim_log2FC <- matrix(NA,nrow=NumIterations,ncol=NumRows)
        Sim_log2var <- matrix(NA,nrow=NumIterations,ncol=NumRows)

        for (l in seq(NumRows)){
            # generate rnorm vector of random null hypothesis sampling
            SimulatedMatrix1 <- abs(rnorm(n = NumIterations*(length(log2Indices1)), sd=sqrt(log2Clusters$var1[rows[l]]),
                                          mean=log2(((rowMeans2(as.matrix(log2Clusters[rows[l],c(log2Indices1,log2Indices2)]))*2)*Gene_R1)+1)))
            SimulatedMatrix2 <- abs(rnorm(n = NumIterations*(length(log2Indices2)), sd=sqrt(log2Clusters$var2[rows[l]]),
                                          mean=log2(((rowMeans2(as.matrix(log2Clusters[rows[l],c(log2Indices1,log2Indices2)]))*2)*Gene_R2)+1)))
            # transform these vectors into matrices
            SimulatedMatrix1 <- matrix(SimulatedMatrix1, nrow = NumIterations)
            SimulatedMatrix2 <- matrix(SimulatedMatrix2, nrow = NumIterations)

            # compute log2FC means and variances from above matrices
            Sim_log2FC[,l] <- rowMeans2(as.matrix(SimulatedMatrix2)) - rowMeans2(as.matrix(SimulatedMatrix1))
            Sim_log2var[,l] <- rowVars(as.matrix(SimulatedMatrix2)) + rowVars(as.matrix(SimulatedMatrix1))
        }

        Nonparam_Clust_Dists <-vapply(seq(NumIterations),Permuted_ES_nullhypo,Sim_log2FC=Sim_log2FC,
                                      DISTANCE=DISTANCE, LINKAGE=LINKAGE, Sim_log2var=Sim_log2var,
                                      NumClusters=NumClusters, NumRows=NumRows,FUN.VALUE = c(Res=0))
        return(Nonparam_Clust_Dists)
    }

    ### function to compute estimated PValues
    estimateNullhypo <- function(NumIterations=NULL, Gene.FC=NULL, Gene_R1=NULL, Gene_R2=NULL,
                              NumRows=NULL, Nullhypo_Dists=NULL){
        ### Run 100 more iterations
        Nullhypo_Dists_temp <- generate_Nullhypo_Dists(NumIterations, Gene.FC, Gene.R1, Gene.R2, NRows)
        # combine these results with previous
        Nullhypo_Dists <- c(Nullhypo_Dists, Nullhypo_Dists_temp)
        # return nullhypo dists
        return(Nullhypo_Dists)
    }

    ### function to estimate p-values
    estimatePVals <- function(Nullhypo_Dists=NULL, Observed_clust_dist=NULL){
        # error function
        f <- ecdf(Nullhypo_Dists)
        # estimate p-value from error function
        PVal <- 1 - f(Observed_clust_dist)
        # return p-value
        return(PVal)
    }

    ### ExCluster progression message
    message("Pre-processing data ...","",sep="\n")

    ### now set up the log2Clusters data frame which will be used to run the analysis
    log2Clusters <- newlog2FC[,seq(4)]

    ### rename log2Clusters columns & rows
    colnames(log2Clusters) <- c("gene","type","log2FC","var")
    rownames(log2Clusters) <- rownames(log2Clusters)
    ### now that we have added the reads to log2Clusters, we can fill in more columns
    # gene_id values without exon bins
    log2Clusters$gene <- gsub("\\:.*","",rownames(newlog2FC))
    # log2FC values
    log2Clusters$log2FC <- rowMeans2(as.matrix(log2(newlog2FC[,Indices2]+1))) -
        rowMeans2(as.matrix(log2(newlog2FC[,Indices1]+1)))
    # log2 variances calculations per condition
    log2Vars1 <- rowVars(as.matrix(log2(newlog2FC[,Indices1]+1)))
    log2Vars2 <- rowVars(as.matrix(log2(newlog2FC[,Indices2]+1)))
    # the log2FC variance for log2Clusters is the sum of the previous 2 variances
    log2Clusters$var <- log2Vars1 + log2Vars2
    # mean reads across conditions (normal space, not log2 space)
    log2Clusters$meanreads <- rowMeans2(as.matrix(newlog2FC))
    # additional metadata columns to be used later
    log2Clusters$clustnum <- NA
    log2Clusters$MaxClust <- 1
    log2Clusters$pval <- 1
    log2Clusters$padj <- 1

    ### now set add in read counts from newlog2FC
    # number of columns in log2Clusters
    NCols <- ncol(log2Clusters)
    # add in count data
    log2Clusters[,seq(NCols,(NCols+ncol(newlog2FC)-1))] <- newlog2FC
    colnames(log2Clusters)[seq(NCols,(NCols+ncol(newlog2FC)-1))] <-
        c(paste("cond1_rep",seq(length(Indices1)),sep=""),paste("cond2_rep",seq(length(Indices2)),sep=""))
    # Index which columns correspond to conditions 1 and 2 in the 'log2Clusters' data frame
    log2Indices1 <- grep("cond1_",colnames(log2Clusters))
    log2Indices2 <- grep("cond2_",colnames(log2Clusters))
    # now we can remove the 'newlog2FC' data frame
    rm(newlog2FC,envir=sys.frame(-1))

    ### now we remove exons with a log2 read variance of > 9 in either condition (very high variances)
    log2Clusters <- log2Clusters[which(log2Vars1 <= 9 & log2Vars2 <= 9),]
    # clean up
    rm(log2Vars1)
    rm(log2Vars2)

    ### Remove exons with 0 variance
    log2Clusters <- log2Clusters[which(log2Clusters$var > 0),]

    ### First remove exon bins with fewer than 4 max reads across all conditions/replicates, for a given exon
    log2Clusters <- log2Clusters[which(rowMaxs(as.matrix(log2Clusters[,c(log2Indices1,log2Indices2)])) >= 4),]

    #### Remove genes with fewer than 4 reads on average in either condition
    log2IDs <- parseGeneIDs(log2Clusters$gene)
    Counter <- 1
    Start <- 1
    RemovalIndices <- vector("list", 1)

    ### run loop to flag genes with fewer than 4 reads in all exons of condition 1 or 2
    for (i in seq(nrow(log2Clusters))){
        nextValue <- min((i+1),nrow(log2Clusters))
        if ((log2IDs[i] != log2IDs[nextValue]) || i == nrow(log2Clusters)){
            # stop at the current gene
            Stop <- i
            # compute the maximum reads in each condition
            Cond1_GeneExp <- max(rowMaxs(as.matrix(log2Clusters[seq(Start,Stop),log2Indices1])))
            Cond2_GeneExp <- max(rowMaxs(as.matrix(log2Clusters[seq(Start,Stop),log2Indices2])))
            # take the minimum of these 2
            Min_GeneExp <- min(Cond1_GeneExp,Cond2_GeneExp)
            # if the minimum is less than 8 reads, remove this gene
            if (Min_GeneExp < 4){
                RemovalIndices[[Counter]] <- seq(Start,Stop)
                Counter <- Counter + 1
            }
            Start <- i+1
        }
    }

    ### now grab indices for all genes flagged for removal (RemovalIndices) above\
    if (length(unlist(RemovalIndices)) > 0){
        RemovalIndices <- unlist(RemovalIndices)
        ### remove these genes from log2Clusters
        log2Clusters <- log2Clusters[-c(RemovalIndices),]
    }

    ### Index log2Clusters row starst & stops for each gene -- greatly speeds up algorithm
    # Set up initial variables for indexing
    log2IDs <- parseGeneIDs(log2Clusters$gene)

    Start <- 1
    RowStart <- NULL
    RowEnd <- NULL
    Counter <- 1

    ### Run indexing loop
    for (i in seq(nrow(log2Clusters))){
        nextValue <- min((i+1),nrow(log2Clusters))
        if ((log2IDs[i] != log2IDs[nextValue]) || i == nrow(log2Clusters)){
            # stop = ith row
            Stop <- i
            # now assign start/stop
            RowStart[Counter] <- Start
            RowEnd[Counter] <- Stop
            # now add 1 to counter & set "Start" to i
            Start <- (i+1)
            Counter <- Counter + 1
        }
    }

    #####################################################################################################
    #############################  PREPARE & RUN CLUSTERING ALGORITHM  ##################################
    #####################################################################################################

    ### grab unique genes
    Genes <- unique(log2Clusters$gene)
    #### Count number of exons per gene
    log2Clusters$NumExons <- NA
    #### ultimately generate permutation tested pvals
    log2Clusters$pval <- NA
    ## do we run calcluations on this gene??
    ## to be used if nrows > 1 or if NbClust successfully runs
    RunClust <- FALSE
    ## random variables
    ClustCount <- 1
    NumClusters <- 1
    ## collect p-values per gene
    GenePVals <- array()
    ComputedPVals <- array()
    ## collect gene names for these arrays
    NamesGenePVals <- array()

    ### add columns with mean and variances for each condition in log2 space
    log2Clusters$mean1 <- rowMeans2(as.matrix(log2(log2Clusters[,log2Indices1]+1)))
    log2Clusters$mean2 <- rowMeans2(as.matrix(log2(log2Clusters[,log2Indices2]+1)))
    log2Clusters$var1 <- rowVars(as.matrix(log2(log2Clusters[,log2Indices1]+1)))
    log2Clusters$var2 <- rowVars(as.matrix(log2(log2Clusters[,log2Indices2]+1)))

    ### now make any row with log2 mean < 0.33 & log2 variance < 0.33 == c(1,0,0)
    # the purpose of this is to eliminate zero read counts & zero variances that may persist
    # do this for condition 1 first
    # grab the total number of cells in the matrix
    num.Cells1 <- length(unlist(log2Clusters[which(log2Clusters$mean1 <= 0.33 & log2Clusters$var1 <= 0.33), log2Indices1]))
    # now make these cells either c(1,0,1) or c(1,0,0)
    if (num.Cells1 > NumReps1){
        log2Clusters[which(log2Clusters$mean1 <= 0.33 & log2Clusters$var1 <= 0.33), log2Indices1] <-
            c(rep(1,num.Cells1/NumReps1),rep(0,num.Cells1/NumReps1),sample(c(0,1),size=(num.Cells1-(2*(num.Cells1/NumReps1))),replace=TRUE))
    }
    # now do the same for condition 2
    num.Cells2 <- length(unlist(log2Clusters[which(log2Clusters$mean2 <= 0.33 & log2Clusters$var2 <= 0.33), log2Indices2]))
    if (num.Cells2 > NumReps2){
        log2Clusters[which(log2Clusters$mean2 <= 0.33 & log2Clusters$var2 <= 0.33), log2Indices2] <-
            c(rep(1,num.Cells2/NumReps2),rep(0,num.Cells2/NumReps2),sample(c(0,1),size=(num.Cells2-(2*(num.Cells2/NumReps2))),replace=TRUE))
    }

    ### now we re-compute log2 means and variances for each condition
    log2Clusters$mean1 <- rowMeans2(as.matrix(log2(log2Clusters[,log2Indices1]+1)))
    log2Clusters$mean2 <- rowMeans2(as.matrix(log2(log2Clusters[,log2Indices2]+1)))
    log2Clusters$var1 <- rowVars(as.matrix(log2(log2Clusters[,log2Indices1]+1)))
    log2Clusters$var2 <- rowVars(as.matrix(log2(log2Clusters[,log2Indices2]+1)))

    ### now make sure all genes with < 2 mean log2 read have minimum variance of 0.33
    if (length(log2Clusters$var1[which(log2Clusters$mean1 < 2)]) > 0){
        log2Clusters$var1[which(log2Clusters$mean1 < 2)] <- 0.33
    }
    if (length(log2Clusters$var2[which(log2Clusters$mean2 < 2)]) > 0){
        log2Clusters$var2[which(log2Clusters$mean2 < 2)] <- 0.33
    }

    ### now we make sure all log2 variances are at least 1e-08 (very low, no variance should be this low)
    log2Clusters$var1 <- log2Clusters$var1 + 0.00000001
    log2Clusters$var2 <- log2Clusters$var2 + 0.00000001

    ### lastly we re-compute the log2FC and log2var for the gene
    log2Clusters$log2FC <- log2Clusters$mean2 - log2Clusters$mean1
    log2Clusters$var <- log2Clusters$var1 + log2Clusters$var2

    ######################################################################################################
    #########################################  NOW RUN EXCLUSTER  ########################################
    ######################################################################################################

    ### ExCluster progression message
    message("Running main ExCluster module (very long) ...","",sep="\n")

    ## set up progress reporting on ExCluster
    ProgressPoints <- round(length(Genes)*c((seq(10))/10))
    ProgressCounter <- 1

    ### loop through and hierarchically cluster all expressed genes with > 1 exon bin
    for (i in seq(length(Genes))){
        # display progress at every 10% of genes analyzed
        if (i == ProgressPoints[ProgressCounter]){
            message(paste("ExCluster progress: ",(ProgressCounter*10),"%",sep=""))
            ProgressCounter <- min((ProgressCounter+1),10)
        }

        # rows of log2Clusters datamatrix for Genes[i]
        rows <- seq(RowStart[i],RowEnd[i])
        # number of rows in Genes[i]
        NRows <- length(rows)
        # number of exons in this gene
        log2Clusters$NumExons[rows] <- NRows

        if (NRows > 1){
            ### Now grab the log2FC means and variances for each exon to make Effect Size difference matrices
            Mean1 <- as.numeric(log2Clusters$log2FC[rows])
            Var1 <- as.numeric(log2Clusters$var[rows])

            ### run Effect Size computation
            ES_mat <- Generate_ES_dists(Mean1,Var1)
            Mean_ES <- mean(abs(unlist(ES_mat)))
            if (Mean_ES == 0){
                NRows <- 1
            }

            if (NRows > 3){
                HC <- hclust(dist(ES_mat,method = DISTANCE),method = LINKAGE)
                HC_cutree <- Cutree_SD.Index(DISTANCE, NRows, HC, ES_mat)
                log2Clusters$clustnum[rows] <- HC_cutree
            }else{
                log2Clusters$clustnum[rows] <- seq(length(rows))
            }
        }else{
            log2Clusters$clustnum[rows] <- 1
        }

        ##Calc the number of clusters in the gene being tested
        NumClusters <- max(log2Clusters$clustnum[rows])
        log2Clusters$MaxClust[rows] <- max(log2Clusters$clustnum[rows])

        if (NumClusters > 1 && NRows > 1){
            ##Generate hclust object
            ES_Dists <- as.matrix(dist(ES_mat,method = DISTANCE))
            HC <- hclust(as.dist(ES_Dists),method=LINKAGE)
            ##cut tree
            HC_cutree <- cutree(HC,k=NumClusters)
            ## find row indices per cluster
            HC_indices <- lapply(seq(NumClusters),FUN=Determine_HC_Distance, HC_cutree=HC_cutree)
            ## find the average euclidean distance between two most different clusters in real observed data
            Observed_clust_dist <- Compare_HC_Distances(NumClusters, ES_Dists, HC_indices)
            # clean up
            rm(HC_indices)
            rm(HC)
            rm(HC_cutree)
            rm(ES_Dists)
            rm(ES_mat)

            ###################  RUN PERMUATIONS PER GENE  ####################

            ## grab overall log2FC of gene for simulation
            gene_log2FC <- log2(colMeans2(as.matrix(log2Clusters[rows,log2Indices2]))+1) -
                log2(colMeans2(as.matrix(log2Clusters[rows,log2Indices1]))+1)
            Gene.FC <- 2^gene_log2FC
            ## gene ratio 1
            Gene.R1 <- 1/(1+Gene.FC)
            ## gene ratio 2
            Gene.R2 <- Gene.FC/(1+Gene.FC)
            ## variable to store distances
            Nullhypo_Dists <- NULL
            ## number of iterations per run, based on p-value
            iteration.Vector <- c(20,30,50,100,300,500)
            ## pvalue cutoffs
            PVal.Cutoffs <- c(0.5, 0.2, 0.05, 0.01, 0.002, 0)
            ## PVal loop variable
            PVal.Loop <- 1

            ### now do a while loop to compute PVals
            while (PVal.Loop >= 1){
                # set the number of iterations
                NumIterations <- iteration.Vector[PVal.Loop]
                # run n null hypothesis hierarchical distance estimates
                Nullhypo_Dists <- estimateNullhypo(NumIterations, Gene.FC, Gene.R1,
                                                   Gene.R2,NRows,Nullhypo_Dists)
                # now estimate the p-values from these null hypothesis distances
                PVal <- estimatePVals(Nullhypo_Dists,Observed_clust_dist)

                ### now determine if we continue
                if (PVal <= PVal.Cutoffs[PVal.Loop]){
                    # do we stop if p-value == 0 ?
                    if (PVal == 0 && PVal.Loop >= 6){
                        # p-value based on gamma function
                        PVal <- gm_mean(pgamma(Observed_clust_dist, Nullhypo_Dists, lower.tail = FALSE)
                                        * gamma(Nullhypo_Dists))
                        PVal <- min(PVal, 0.0005)
                        break
                    }
                    # add one to PVal.Loop
                    PVal.Loop <- PVal.Loop + 1
                }else{
                    # lastly if PVal >= PVal.Cutoffs
                    PVal.Loop <- 0
                }
            }

            # clean up
            rm(Nullhypo_Dists)
            rm(Observed_clust_dist)
            # assign pvals & Gene Names for stats
            log2Clusters$pval[rows] <- PVal
        }
        rm(NRows)
    }
    return(log2Clusters)
}

ExClust_compute_stats <- function(gene.IDs=NULL, gene.PVals=NULL){

    ### first we need to grab the gene names & p-values for each gene
    # grab the 1st row of each gene
    pvalue.Indices <- match(unique(gene.IDs), gene.IDs)
    # now grab the gene IDs from these rows
    NamesGenePVals <- gene.IDs[pvalue.Indices]
    # lastly grab the p-values from these rows
    GenePVals <- gene.PVals[pvalue.Indices]

    ### ExCluster progression message
    message("Running statistical tests ...","",sep="\n")

    ### data frame for testing p-values
    Catch_PVals <- data.frame(NamesGenePVals,GenePVals)
    colnames(Catch_PVals) <- c("EnsG","pvals")
    # remove NAs
    Catch_PVals <- Catch_PVals[which(complete.cases(Catch_PVals[,2])),]
    # sort PVals from lowest to highest
    Catch_PVals <- Catch_PVals[order(Catch_PVals[,2]),]

    if (nrow(Catch_PVals) >= 5000){
        ### normalize p-values
        # estimate the number of 'truly spliced genes'
        pvalCutoff <- EstNullHypo(Catch_PVals[,2])+0.005
        # estimate the number of truly spliced genes (may include some false positives)
        NumTrue <- length(which(Catch_PVals[,2] <= pvalCutoff))
        # estimate the number of null hypothesis genes
        NumNull <- length(which(Catch_PVals[,2] > pvalCutoff))

        ### if we have > 0 NumTrue p-values
        if (NumTrue > 0){
            # Now give the null hypothesis p-values uniform distriubtions
            Catch_PVals[seq((NumTrue+1),(NumTrue+NumNull)),2] <- sort(runif(n=NumNull,0,1))
            # Now divide all true p-values by a factor based on the lowest null hypothesis value
            Catch_PVals[seq(NumTrue),2] <- Catch_PVals[seq(NumTrue),2]/
                                            (Catch_PVals[NumTrue,2]/Catch_PVals[(NumTrue+1),2])
            # Now compute FDR values
            FDRs <- FDRcalc(Catch_PVals[,2],NumTrue,NumNull)
        }else{
            ### if we have no truly significant p-values from our estimate
            Catch_PVals[,2] <- sort(runif(n=NumNull,0,1))
            FDRs <- rep(1,nrow(Catch_PVals))
        }

    }else{

        ### This part of the code will be run if less than 5000 expressed genes were identified
        warning(call = ExCluster_errors$too_few_pvalues)

        # because we have too few pvalues (< 5000), we cannot run our advanced stats
        # we therefore run a basic correction with Benjamini-Hochberg
        pvalCutoff <- 0.025
        # estimate the number of truly spliced genes
        NumTrue <- nrow(Catch_PVals[Catch_PVals[,2] <= pvalCutoff,])
        # estimate the number of null hypothesis genes
        NumNull <- nrow(Catch_PVals[Catch_PVals[,2] > pvalCutoff,])
        # set the minimum value a null hypothesis p-value can be corrected to
        minNullPVal <- 0.005

        ### if we have truly differentially spliced genes
        if (NumTrue > 0){
            # Now give the null hypothesis p-values uniform distriubtions
            Catch_PVals[seq((NumTrue+1),(NumTrue+NumNull)),2] <- sort(runif(n=NumNull,minNullPVal,1))
            # Now divide all true p-values by a factor based on the lowest null hypothesis value
            Catch_PVals[seq(NumTrue),2] <- Catch_PVals[seq(NumTrue),2]/(Catch_PVals[NumTrue,2]/minNullPVal)
            # Now compute FDR values
            FDRs <- p.adjust(p = Catch_PVals[,2],method="BH")
        }else{
            ### if we have no truly significant p-values from our estimate
            Catch_PVals[,2] <- sort(runif(n=NumNull,0,1))
            FDRs <- rep(1,nrow(Catch_PVals))
        }
    }

    ### Make statistics table per gene with pvals and FDRs
    Stats_table <- data.frame(Catch_PVals,FDRs)
    return(Stats_table)
}

GFF_convert <- function(annot.GTF=NULL, GTF.File=NULL,GFF.File=NULL){

    ### make sure R doesn't add factors -- R creators never should have made this default = TRUE
    options(stringsAsFactors=FALSE)

    ### Check to ensure that the GTF variable was given, GFF variable is optional
    if (is.null(GTF.File) == TRUE){
        if (is.null(annot.GTF) == TRUE){
            stop(call = ExCluster_errors$GTF.File_missing)
        }
    }

    ### Now check that the GTF file path specified actually exists if annot.GTF == false
    if (is.null(annot.GTF) == TRUE){
        if (file.exists(GTF.File) == FALSE){
            stop(call = ExCluster_errors$bad_GTF_filepath)
        }else{
            # if the GTF.File exists, read it into R
            gtf.data<-read.table(GTF.File, header=FALSE, sep="\t", stringsAsFactors=FALSE)
        }
    }else{

        ### if annot.GTF exists, read it into R
        # if the typeof(annot.GTF) is S4, treat it as an rtracklayer object
        if (typeof(annot.GTF) == "S4"){
            # reformat rtracklayer to GTF
            gtf.data <- reformat_GTF(rtracklayer.GTF=annot.GTF)
        }else{
            # rename annot.GTF
            gtf.data <- annot.GTF
        }
        # clean up
        rm(annot.GTF)
    }

    ######################## Check GTF file for correct formatting ########################

    ### run some checks to verify the GTF file format integrity

    ### double check to make sure that 9 columns exactly are present in the GTF file
    if (ncol(gtf.data) != 9){
        stop(call = ExCluster_errors$check_GTF_columns)
    }

    ### check to make sure column 3 has values 'gene' 'exon' and 'transcript'
    feature.Values <- unique(gtf.data[,3])
    # check to see if "gene", "transcript", and "exon" are present in feature.Values
    feature.Indices <- match(c("gene","transcript","exon"),feature.Values)
    # the length of the above should be 3
    if (length(feature.Indices) != 3){
        stop(call = ExCluster_errors$GTF_missing_features)
    }

    ### check to make sure columns 4 & 5 are numeric start/stops with positive differences only
    # we do this by subtracting 'start' from 'stop' and making sure the math works, and minimum = 0
    # test to see if the math will work (will only work if columns 4 & 5 are properly formatted)
    coord.Diff <- try(as.numeric(gtf.data[,5]) - as.numeric(gtf.data[,4]),silent = TRUE)
    # check to see if we have an error
    if (substr(coord.Diff[1],1,5) == "Error"){
        stop(call = ExCluster_errors$bad_GTF_start_stop)
    }
    # if that did not fail, the code continues and we verify that all stop - start subtractions are >= 0
    if (min(coord.Diff) < 0){
        stop(call = ExCluster_errors$start_stop_not_numeric)
    }

    ### check to make sure column 7 unique values have at least "+" or "-" (could be just one for test gtf files)
    # unique values of column 7 in the GTF file
    strand.Values <- unique(gtf.data[,7])
    # is "+" present? if true this will == 1, if false this will == 0
    positive.Length <- length(which('+'%in%strand.Values))
    # is "-" present? if true this will == 1, if false this will == 0
    negative.Length <- length(which('-'%in%strand.Values))
    # make sure there are no more than 2 unique values of column 7
    if (length(strand.Values) > 2){
        stop(call = ExCluster_errors$bad_GTF_plus_minus)
    }
    # if the previous check passed, make sure at least one of '-' or '+' is present
    if ((positive.Length + negative.Length) < 1){
        stop(call = ExCluster_errors$GTF_plus_minus_missing)
    }

    ### lastly, check to make sure transcript_id and gene_id values are present in GTF metadata column 9
    # grab 1st 25 rows -- sufficient for testing purposes
    metacol.Values <- gtf.data[seq(1,25),9]
    # check the number of rows (if any) 'gene_id' is present in for this metadata column
    gene_id.Length <- length(grep("gene_id",metacol.Values))
    # check the number of rows (if any) 'transcript_id' is present in for this metadata column
    transcript_id.Length <- length(grep("transcript_id",metacol.Values))
    # now make sure gene_id.Length == 25 and transcript_id.Length > 1
    # this is because each row should have a gene_id, and at least some rows should contain a transcript_id
    if (gene_id.Length != 25 || transcript_id.Length < 1){
        stop(call = ExCluster_errors$GTF_missing_gene_id)
    }

    #########################################################################################################
    ########################################### MAIN CODE ###################################################
    #########################################################################################################

    # select only GTF rows that correspond to exons (includes UTRs, CDS, and retained introns)
    gtf.data<-gtf.data[which(gtf.data$V3 =='exon'),]
    # add columns to gtf.file for gene_name, gene_id, and transcript_id
    gtf.data$gene.id <- gsub(".*gene_id (.*?);.*", "\\1", gtf.data$V9)
    gtf.data$gene.name <- gsub(".*gene_name (.*?);.*", "\\1", gtf.data$V9)
    gtf.data$tx.id <- gsub(".*transcript_id (.*?);.*", "\\1", gtf.data$V9)
    # now remove that mess of a 9th column
    gtf.data <- gtf.data[,-c(9)]
    # make an empty data matrix for gff file
    gff.data<- vector("list", 1)
    # generate a hash table for the start/stop of each gene (2nd and 3rd columns, 1st column gene_id)
    GeneHash <- IndexGeneStartStop(gtf.data$gene.id)
    # now loop through each gene in the GeneHash table & collapse exon bins with an lapply function
    for (x in seq(nrow(GeneHash))){
        # subset the current gene data
        temp.data <- gtf.data[seq(GeneHash[x,2],GeneHash[x,3]),]
        # collapse exon bins for this gene
        exons.collapsed <- CollapseExons(temp.data)
        # exon bin numbers for the current gene (1:n for +ve strand by default, n:1 for -ve strand)
        Bins <- seq(nrow(exons.collapsed))
        # strand is by default +ve
        chr.Strand <- "+"
        # if -ve strand, reverse this
        if (temp.data[1,7] == "-"){
            Bins <- rev(Bins)
            # also change chr.Strand to -ve
            chr.Strand <- "-"
        }
        # make temporary gff matrix
        temp.gff <- matrix(NA,nrow=nrow(exons.collapsed),ncol=9)
        # add chromosome names
        temp.gff[,1] <- exons.collapsed[,1]
        # add EnsID + exon bin
        temp.gff[,2] <- "."
        # add exon_bin column
        temp.gff[,3] <- "exon"
        # add start + stop columns
        temp.gff[,c(4,5)] <- exons.collapsed[,c(3,4)]
        # add '.' columns
        temp.gff[,c(6,8)] <- "."
        # add strand column (+ve or -ve based on previous chr.Strand)
        temp.gff[,7] <- chr.Strand
        # prepare gene IDs, Names, and Transcripts
        gff.ID <- sprintf(paste("ID=",exons.collapsed[,2],":%03d",sep=""),Bins)
        gff.Name <- sprintf(paste("Name=",exons.collapsed[,6],sep=""))
        gff.Transcript <- paste("Transcripts=",exons.collapsed[,5],sep="")
        # add a final 9th column with gene IDs, names, and transcripts
        temp.gff[,9] <- paste(gff.ID,gff.Name,gff.Transcript,sep=";")
        rm(gff.ID)
        rm(gff.Name)
        rm(gff.Transcript)
        # assign temp.gff to be item x on the gff.data list
        gff.data[[x]] <- temp.gff
    }
    # clean up gtf data now
    rm(gtf.data)
    # now we can combine the gff data into one data frame structure with do.call
    gff.data <- data.frame(do.call(rbind,gff.data))
    # change filenames to arbitrary V1, V2, etc. (necessary for next step in read counting)
    colnames(gff.data) <- c("V1","V2","V3","V4","V5","V6","V7","V8","V9")
    # parse out IDs from the GFF data
    #gff3.IDs <- gsub(".*ID=(.*?);.*", "\\1", gff.data[,9])
    # sort on position and then chromsome
    #gff.data <- gff.data[order(gff.data[,1],gff3.IDs),]

    ### if the GFF outpath was specified, AND the filepath is writeable, write out the file
    # check if file path is writeable
    if (is.null(GFF.File) == FALSE){
        WriteCheck <- file.access(dirname(GFF.File), mode=2)
        if (WriteCheck == 0){
            write.table(gff.data, file=GFF.File,sep="\t",quote=FALSE,col.names=FALSE,row.names=FALSE)
        }
    }

    # reformat GFF3 data to GRanges object
    gff.data <- GRangesFromGFF(gff.data)
    # return & end function
    return(gff.data)
}

ExCluster <- function(exon.Counts=NULL, cond.Nums=NULL, annot.GFF = NULL, GFF.File=NULL,
                      out.Dir=NULL, result.Filename=NULL, combine.Exons=TRUE,
                      plot.Results=FALSE, FDR.cutoff=0.05){

    ### make sure R doesn't add factors -- R creators never should have made this default = TRUE
    options(stringsAsFactors=FALSE)

    ### ExCluster progression message
    message("","Initializing ...",sep="\n")

    ### check to make sure a normalized exon count matrix was provided as input
    if (is.null(exon.Counts) == TRUE){
        stop(call = ExCluster_errors$exon_counts_missing)
    }

    ### check to make sure condition numbers were specified
    if (is.null(cond.Nums) == TRUE){
        stop(call = ExCluster_errors$cond_nums_missing)
    }

    ### ensure the number of columns in the exon.Counts parameter equal the number of cond.Nums
    if (ncol(exon.Counts) != length(cond.Nums)){
        stop(call = ExCluster_errors$exon_count_length_incorrect)
    }

    ## grabbing unique condition numbers (should only ever be two conditions)
    uCondNums <- unique(cond.Nums)

    ## ensure exactly 2 conditions
    if (length(uCondNums) != 2){
        stop(call = ExCluster_errors$improper_cond_nums)
    }

    ## condition indices
    Indices1 <- grep(uCondNums[1],cond.Nums)
    Indices2 <- grep(uCondNums[2],cond.Nums)

    ## Ensure that both conditions have at least 2 replicates provided, or ExCluster will not function.
    if (length(Indices1) == 1){
        stop(call = ExCluster_errors$not_enough_replicates_cond1)
    }
    if (length(Indices2) == 1){
        stop(call = ExCluster_errors$not_enough_replicates_cond2)
    }

    ### check to ensure that there are no duplicated data columns in exon.Counts
    CheckDups <- duplicated(t(exon.Counts))
    if (any(CheckDups) == TRUE){
        DupCols <- which(CheckDups == TRUE)
        message(paste("Columns",DupCols,"were duplicated in your normalized exon count data.
    Please re-check your BAM file names and re-run processCounts."))
        stop(call = ExCluster_errors$exon_read_counts_duplicated)
        }

    ### check to make sure either a GFF data structure or GFF file path were specified
    if (is.null(annot.GFF) == TRUE){
        if (is.null(GFF.File) == TRUE){
            stop(call = ExCluster_errors$GFF_annotations_missing)
        }else{
            ### Test if GFF.File works
            if(file.exists(GFF.File) != TRUE){
                stop(call = ExCluster_errors$GFF_file_inaccessible)
            }
        }
    }

    #####################################################################################################
    #################################  READ COUNT & DATA PREPARATION  ###################################
    #####################################################################################################

    ### ExCluster progression message
    message('','Reading in data ...','',sep="\n")

    log2FC <- data.frame(exon.Counts)
    rm(exon.Counts)

    ## Grab number of replicates from Cond1 and Cond2
    NumReps1 <- length(Indices1)
    NumReps2 <- length(Indices2)

    ######################### READ IN GFF FILE IF NECESSARY #########################

    ### Read in the GFF file if the annot.GFF argument was not specified
    if (is.null(annot.GFF) == TRUE){
        if (is.null(GFF.File) == FALSE){
            ### Read in GFF file and grab exon bins (THIS MUST BE THE EXACT GFF FILE USED TO GENERATE READ COUNTS!!!)
            annot.GFF <- read.table(file=GFF.File,sep="\t",stringsAsFactors=FALSE)
        }
    }else{
        ### now check to see if annot.GFF is an S4 object (GRanges) & reformat it
        if (substr(typeof(annot.GFF),1,2) == "S4"){
            annot.GFF <- GRangesToGFF(annot.GFF)
        }
    }

    ######################### NOW VERIFY GFF FILE FORMAT INTEGRITY #########################

    if (ncol(annot.GFF) != 9){
        ### inform the user that the GFF file was not properly formatted
        stop(call= ExCluster_errors$ExClust_GFF_bad_colnum)
    }

    ### check to make sure that each element of column 3 in the GFF file begins with 'exon'
    exon_feature.Values <- unique(annot.GFF[,3])
    # if the length of this unique value is > 1, or the result does not == 'exon', throw an error
    if (length(exon_feature.Values) > 1){
        stop(call = ExCluster_errors$GFF_missing_exon_field)
    }

    ### check to make sure columns 4 & 5 are numeric start/stops with positive differences only
    # we do this by subtracting 'start' from 'stop' and making sure the math works, and minimum = 0
    # test to see if the math will work (will only work if columns 4 & 5 are properly formatted)
    coord.Diff <- try(as.numeric(annot.GFF[,5]) - as.numeric(annot.GFF[,4]),silent = TRUE)
    # check to see if we have an error
    if (substr(coord.Diff[1],1,5) == "Error"){
        stop(call = ExCluster_errors$GFF_numeric_error)
        }
    # if that did not fail, the code continues and we verify that all stop - start subtractions are >= 0
    if (min(coord.Diff) < 0){
        stop(call = ExCluster_errors$GFF_negative_dists)
    }

    ### check to make sure column 7 unique values have at least "+" or "-" (could be just one for test GFF files)
    # unique values of column 7 in the GFF file
    strand.Values <- unique(annot.GFF[,7])
    # is "+" present? if true this will == 1, if false this will == 0
    positive.Length <- length(which('+'%in%strand.Values))
    # is "-" present? if true this will == 1, if false this will == 0
    negative.Length <- length(which('-'%in%strand.Values))
    # make sure there are no more than 2 unique values of column 7
    if (length(strand.Values) > 2){
        stop(call = ExCluster_errors$bad_GFF_strand_column)
    }
    # if the previous check passed, make sure at least one of '-' or '+' is present
    if ((positive.Length + negative.Length) < 1){
        stop(call = ExCluster_errors$GFF_plus_minus_missing)
    }
    # now check to make sure that column 9 contains 'ID=', 'Name=', and 'Transcripts=' strings
    ID.indices <- length(grep(pattern = "ID=", as.character(annot.GFF[seq(2),9])))
    Name.indices <- length(grep(pattern = "Name=", as.character(annot.GFF[seq(2),9])))
    Transcripts.indices <- length(grep(pattern = "Transcripts=", as.character(annot.GFF[seq(2),9])))
    # if the lengths of these indices do not sum to 6, throw an error
    if (ID.indices+Name.indices+Transcripts.indices != 6){
        stop(call=ExCluster_errors$GFF_missing_GFF3_fields)
    }

    ### if all of these checks passed, reformat the GFF3 formatted file into the prefered GFF format
    annot.GFF <- reformat_GFF3(annot.GFF)

    ######################### NOW BEAT DATA INTO SHAPE FOR log2FC and annot.GFF #########################

    # first sort log2FC on EnsG:exon_bin
    log2FC <- log2FC[order(rownames(log2FC)),]
    # do the same for annot.GFF
    annot.GFF <- annot.GFF[order(annot.GFF$V2),]

    # now copy log2FC to newlog2FC, so we can work with newlog2FC and preserve log2FC
    newlog2FC <- log2FC

    ## now assign Gene names and exon numbers for later joining to log2FC -- used for plotting exons
    log2Bins <- gsub(pattern = '.*\\:',replacement = "",x = rownames(log2FC))
    GFF.Chr <- annot.GFF$V1
    GFF.Strand <- annot.GFF$V7
    GFF.Start <- annot.GFF$V4
    GFF.End <- annot.GFF$V5

    ### Now fill in log2FC data for later use in output
    # now assign Genes & Bins to log2FC
    log2FC$Genes <- annot.GFF$V3
    log2FC$Bins <- log2Bins

    # compute means and variances for the log2FC data frame
    log2FC$log2FC <- rowMeans2(as.matrix(log2(log2FC[,Indices2]+1))) -
        rowMeans2(as.matrix(log2(log2FC[,Indices1]+1)))
    log2FC$log2Variance <- rowVars(as.matrix(log2(log2FC[,Indices2]+1))) +
        rowVars(as.matrix(log2(log2FC[,Indices1]+1)))

    ### now we work on organizing the newlog2FC read count data
    # remove all exon bin rows with fewer than 2 max reads (log2 reads >= 1) across samples
    newlog2FC <- newlog2FC[which(rowMaxs(as.matrix(newlog2FC)) >= 1),]
    # now add EnsID to remove duplicated rows
    newlog2FC$EnsID <- sub(":.*","",rownames(newlog2FC))
    # identify duplicated rows by condition means
    dupRows <- which(duplicated(newlog2FC))

    # remove duplicated rows if there are any
    if (length(dupRows) > 0){
        newlog2FC <- newlog2FC[-c(dupRows),]
    }
    # remove EnsiD column
    newlog2FC <- newlog2FC[,-c(ncol(newlog2FC))]
    # set up newlog2FC "Bins"
    newBins <- sub(".*:","",rownames(newlog2FC))

    # take Ensembl ID numbers for easy parsing of newlog2FC data
    newIDs <- parseGeneIDs(gene.Annot =  rownames(newlog2FC))

    ### Add Ensembl ID column to annot.GFF data frame for later sorting
    annot.GFF$EnsID <- parseGeneIDs(gene.Annot = annot.GFF[,2])

    ### remove ENSG00000 from ENSEMBL IDs to make them easier to sort (for looping through each unique ID)
    IDs <- parseGeneIDs(gene.Annot = rownames(log2FC))

    ### Check to make that annot.GFF EnsID column and IDs vector are 100% identical
    EqualVectors <- all.equal(annot.GFF$EnsID,IDs)
    if (EqualVectors != TRUE){
        stop(call=ExCluster_errors$match_GFF_counts_failed)
    }

    ###############################  IF TRUE, COMBINE COMMON EXONS  #####################################

    if (combine.Exons == TRUE){
        ### ExCluster progression message
        message("Combining common exons ...","",sep="\n")

        # Count number of rows in annot.GFF
        NROW_GFF <- nrow(annot.GFF)
        # Counter for counting new summed exon bins
        Counter <- 1
        # Start point for first newIDs gene row number
        newIDstart <- 1
        # Variable start will be used through loop
        Start <- 1
        # make newlog2FC list to dump results from combine.Exons into
        temp_log2FC <- vector("list", 1)

        for (i in seq(NROW_GFF)){
            nextValue <- min((i+1), NROW_GFF)
            ### check if we are on a new gene
            if ((IDs[i] != IDs[nextValue]) == TRUE | i == NROW_GFF){

                # stop is the current ith trow
                Stop <- i
                # Rows of the log2FC data frame
                log2Rows <- seq(Start,Stop)

                ### now determine if the new gene is in newIDs
                if ((IDs[Start] == newIDs[newIDstart]) == TRUE){
                    # temporary newlog2FC data
                    TempIDs <- newIDs[seq(newIDstart,(newIDstart+length(log2Rows)))]
                    # determine the Stop point of newIDs
                    newIDstop <- newIDstart+length(which(TempIDs%in%IDs[Start]))-1
                    # Rows of the newIDs vector
                    newlog2Rows <- seq(newIDstart,newIDstop)
                    # grab all transcript combinations and take unique transcript combinations per gene
                    ExTrans <- annot.GFF[log2Rows,9]
                    # now grab only rows of log2FC matching & present in newlog2FC
                    matchingRows <- which(rownames(log2FC)[log2Rows]%in%rownames(newlog2FC)[newlog2Rows])
                    # now select only those elements from ExTrans based on the matchingRows
                    ExTrans <- ExTrans[c(matchingRows)]
                    # grab unique transcript combinations
                    uExTrans <- unique(ExTrans)
                    # grab original exon bins for the gene from log2FC
                    originalBins <- gsub(pattern = '.*\\:',replacement = "",x = rownames(log2FC)[log2Rows])
                    # now grab the new bins based on matching rows
                    newBins <- originalBins[matchingRows]

                    ### now run the combining exons script for this newlog2FC gene
                    if (length(uExTrans) > 1){
                        ## Make temporary read table for summing reads
                        TempReads <- newlog2FC[newlog2Rows,,drop=FALSE]
                        ### Make a matrix that will represent the presence or absence of transcripts
                        TransMatrix <- data.frame(matrix(data=0,nrow=length(uExTrans),ncol=ncol(TempReads)))
                        for (j in seq(length(uExTrans))){
                            uIndices <- which(ExTrans%in%uExTrans[j])
                            if (length(uIndices) > 1){
                                TransMatrix[j,] <- colSums2(as.matrix(TempReads[uIndices,]))
                                rownames(TransMatrix)[[j]] <- rownames(TempReads)[[uIndices[1]]]
                                # match new bins onto original log2FC rows
                                newBinIndices <- which(originalBins%in%newBins[uIndices])
                                # now make those original log2FC rows equal to the 1st newBins in uIndices
                                log2FC$Bins[newBinIndices] <- newBins[uIndices[1]]
                            }else{
                                TransMatrix[j,] <- TempReads[uIndices,]
                                rownames(TransMatrix)[[j]] <- rownames(TempReads)[[uIndices[1]]]
                                # match new bins onto original log2FC rows
                                newBinIndices <- which(originalBins%in%newBins[uIndices[1]])
                                # now make those original log2FC rows equal to the 1st newBins in uIndices
                                log2FC$Bins[newBinIndices] <- newBins[uIndices[1]]
                            }
                        }
                        # add the new TransMatrix to temp_log2FC if uExTrans > 1
                        temp_log2FC[[Counter]] <- TransMatrix
                        # increase the counter
                        Counter <- Counter + 1
                    }

                    ### now set up the newIDstart to match the next gene in the list, but not > length(newIDs)
                    newIDstart <- min((newIDstop+1),length(newIDs))
                }
                if (i != NROW_GFF){
                    # now start the next gene
                    Start <- i+1
                }
            }
        }
        newlog2FC <- do.call(rbind,temp_log2FC)
        # clean up
        rm(temp_log2FC)
    }
    # clean up annot.GFF
    rm(annot.GFF)
    rm(log2Bins)
    rm(IDs)
    rm(newIDs)

    ################################### Now run ExCluster main function ##################################

    log2Clusters <- ExClust_main_function(newlog2FC, Indices1, Indices2, NumReps1, NumReps2)

    #################################### Compute ExCluster statistics ####################################

    Stats_table <- ExClust_compute_stats(log2Clusters$gene, log2Clusters$pval)

    ######################################################################################################
    ###############################  FINAL DATA PREPARATION FOR OUTPUT ###################################
    ######################################################################################################

    ### ExCluster progression message
    message("Final ExCluster data organization & output ...","",sep="\n")

    # set up statistics & cluster columns in log2FC
    log2FC$pval <- NA
    log2FC$FDR <- NA
    log2FC$Cluster <- NA

    ### remove ENSG00000 from ENSEMBL IDs to make them easier to sort (for looping through each unique ID)
    # Now add an IDs column to the log2FC data frame
    log2FC$IDs <- parseGeneIDs(gene.Annot = rownames(log2FC))
    # now do the same for the IDs in the stats table
    Stats_table$IDs <- parseGeneIDs(Stats_table[,1])
    # now do the same for IDs in the log2Clusters
    log2Clusters$IDs <- parseGeneIDs(log2Clusters$gene)
    # Also add the exon bin number to log2Clusters for matching
    log2Clusters$Bin <- gsub(pattern = '.*\\:',replacement = "",x = rownames(log2Clusters))

    # now order on EnsG
    log2FC <- log2FC[order(rownames(log2FC)),]
    Stats_table <- Stats_table[order(Stats_table$EnsG),]
    log2Clusters <- log2Clusters[order(rownames(log2Clusters)),]

    # set up counter for Stats_table genes
    StatsCounter <- 1
    # set up a Start location counter for the log2Clusters dataframe
    log2Start <- 1
    # Start location for first gene
    Start <- 1

    ### now run through log2Clusters & Stats_table and match gene data, transfering data to log2F for output
    for (i in seq(2,nrow(log2FC))){

        ### First determine if we have ended the current gene of interest
        if ((log2FC$IDs[i] != log2FC$IDs[(i-1)]) == TRUE | i == nrow(log2FC)){
            # Now determine the stop location for the current gene
            if (i == nrow(log2FC)){
                Stop <- i
            }else{
                Stop <- (i-1)
            }

            ### Now check if the Stats_table ID is equal to the current log2FC gene ID
            if (log2FC$IDs[Start] == Stats_table$IDs[StatsCounter]){
                # now assign stats data to the log2FC data frame, on rows Start:Stop
                log2FC$pval[seq(Start,Stop)] <- Stats_table$pvals[StatsCounter]
                log2FC$FDR[seq(Start,Stop)] <- Stats_table$FDRs[StatsCounter]
                StatsCounter <- min((StatsCounter+1), nrow(Stats_table))
            }

            ### Now check if the log2Clusters ID is equal to the current log2FC gene ID
            if (log2FC$IDs[Start] == log2Clusters$IDs[log2Start]){
                # if above is TRUE, grab data for the next 300 rows in log2Clusters
                # make sure we don't run out of rows in log2Start
                Upper <- min(300,(nrow(log2Clusters)-log2Start))
                TempData <- log2Clusters[seq(log2Start,(log2Start+Upper)),]
                # isolate data for only the current gene
                TempData <- TempData[which(TempData$IDs%in%TempData$IDs[1]),]
                # grab the exon bins from this gene
                ExonBins <- TempData$Bin
                # now loop through the exon bins in TempData & assign cluster numbers to log2FC
                for (j in seq(length(ExonBins))){
                    BinIndices <- which(log2FC$Bins[seq(Start,Stop)]%in%ExonBins[j])
                    log2FC$Cluster[seq(Start,Stop)][BinIndices] <- TempData$clustnum[j]
                }
                # Now set log2Start to the beginning of the next log2Clusters gene
                log2Start <- min(log2Start+length(ExonBins), nrow(log2Clusters))
            }

            ### Reset the gene start for the next log2FC gene
            Start <- i
        }
    }

    # clean up
    rm(log2Clusters)
    rm(Stats_table)

    # separate EnsG and exon bin number
    ExonBins <- gsub(pattern = '.*\\:',replacement = "",x = rownames(log2FC))
    EnsG <- gsub(pattern = '\\:.*',replacement = "",x = rownames(log2FC))

    ### now re-arrange log2FC columns to be more logical
    final.log2FC <- data.frame(EnsG, ExonBins ,log2FC$Genes,log2FC$Cluster,log2FC$log2FC,
                               log2FC$log2Variance,log2FC$pval,log2FC$FDR,GFF.Chr,GFF.Strand,GFF.Start,GFF.End)
    # rename columns
    colnames(final.log2FC) <- c("EnsG","Exon_bin","Gene_name","Cluster",
                                "log2FC","log2Var","pval","FDR","Chr","Strand","Start","End")
    # re-add the read counts at the end of the dataframe
    final.log2FC <- data.frame(final.log2FC,log2FC[,seq(length(cond.Nums))])
    # clean up log2FC
    rm(log2FC)

    ### if out.Dir was specified, write out ExCluster results
    if (is.null(out.Dir) == FALSE){
        # try to write the out.Dir
        try(dir.create(out.Dir, recursive=TRUE, showWarnings = FALSE), silent=TRUE)
        # now check the file directory can be written to
        WriteCheck <- file.access(out.Dir, mode=2)
        # now proceed if WriteCheck == 0
        if (WriteCheck == 0){
            # now check to see if a result.Filename was specified
            if (is.null(result.Filename) == TRUE){
                # assign full filename
                fullFilename <- paste(out.Dir,"/ExClust_Results",sep="")
            }else{
                fullFilename <- paste(out.Dir,"/",result.Filename,sep="")
            }
            # make sure we're not over-writing an existing file
            if (file.exists(paste(fullFilename,".txt",sep="")) == TRUE){
                fullFilename <- paste(fullFilename,Sys.time(),sep="")
            }
            # try to write the file no matter what
            try(write.table(final.log2FC,file=paste(fullFilename,".txt",sep=""),sep="\t",
                            quote=FALSE,row.names=FALSE,col.names=TRUE), silent=TRUE)
            # out.Dir works
            outdir.Check <- TRUE
        }else{
            message(ExCluster_errors$ExClust_bad_outDir)
            # out.Dir unavailable to write to
            outdir.Check <- FALSE
        }

        ### Now see if we need to plot images
        if (plot.Results == TRUE){
            if (outdir.Check == TRUE && is.null(out.Dir) == FALSE){
                # set plot.Type to NULL to begin with
                plot.Type <- "bitmap"
                # new images folder to write to
                out.Dir <- paste(out.Dir,"/exon_log2FC_images/",sep="")
                # try to make out.Dir no matter what
                try(dir.create(out.Dir, recursive=TRUE, showWarnings = FALSE), silent=TRUE)
                # now check the file directory can be written to
                WriteCheck <- file.access(out.Dir, mode=2)
                # if this check passes, proceed
                if (WriteCheck == 0){
                    # make sure that FDR.cutoff is less than 0.2, or set it back to 0.05
                    if (FDR.cutoff > 0.2){
                        warning(call = ExCluster_errors$FDR_cutoff_too_high)
                        FDR.cutoff <- 0.05
                    }
                    # now test to make sur we can plot PNG and/or bitmap
                    test.bitmap <- substr(testBMplot(out.Dir)[1],1,5)
                    # if one of them passes, run the plot
                    if (test.bitmap == "Error"){
                        # try to plot PNG
                        test.PNG <- substr(testPNGplot(out.Dir)[1],1,5)
                        plot.Type <- "PNG"
                        # check if PNG plot succeeds
                        if (test.PNG == "Error"){
                            # plotting has failed
                            plot.Check <- FALSE
                            warning(call = ExCluster_errors$plot_type_failure)
                        }else{
                            # plotting for "PNG" succeeded
                            plot.Check <- TRUE
                        }
                    }else{
                        # plotting for "bitmap" succeeded
                        plot.Check <- TRUE
                    }
                    # run plotting function if plot.Type is not NULL
                    if (is.null(plot.Type) == FALSE && plot.Check == TRUE){
                        plotExonlog2FC(results.Data=final.log2FC, out.Dir=out.Dir,
                                       FDR.cutoff=FDR.cutoff, plot.Type=plot.Type)
                    }
                }
            }else{
                warning(call = ExCluster_errors$plot_results_no_outDir)
            }
        }

    }

    ### return final data frame resutls
    return(final.log2FC)
}

%\VignetteEngine{utils::Sweave}
%\VignetteIndexEntry{ExCluster Vignette}
%\VignetteKeywords{Differential splicing RNA-seq}
%\VignettePackage{ExCluster}

\documentclass[12pt]{article}

% special font for R code to make examples more obvious
\newcommand{\R}[1]{{\texttt{#1}}}

% document parameters
\textwidth=6.3in
\textheight=8.5in
\oddsidemargin=0.2in
\evensidemargin=0.2in
\headheight=0in
\headsep=0in

\begin{document}
\SweaveOpts{concordance=TRUE}

% add title
\title{ExCluster package: Robust detection of differentially spliced genes in RNA-seq data}
\author{R. Matthew Tanner, William L. Stanford, and Theodore J. Perkins}
\date{2 October 2018}
\maketitle
\tableofcontents

\section{Introduction}
This vignette will detail the basic usage of the ExCluster package, both quickly and step-by-step. IMPORTANT NOTE: ExCluster will not function with a Windows operating system, and will only install on linux and MacOSX machines. Unfortunately, this package requires Rsubread, which is not availble for Windows.\\

Before proteins may be produced from genes in our DNA, those genes are first transcribed into pre-messenger RNA (pre-mRNA). This pre-mRNA contains introns and exons, from which introns must be removed to produce mature mRNA; this process is called mRNA splicing. However, not all exons need be combined in the mature mRNA -- some exons may be spliced out in different combinations. This process is known as alternative splicing, and it allows a single gene encode multiple protein coding transcripts, with roughly 20,000 protein coding genes producing nearly 200,000 protein coding transcripts.\\

Different cell types within a given organism may express different levels of mRNA transcripts, and may even express entirely different mRNA transcripts. When two different biological conditions, such as two cell types or two tissues, display differential transcript expression patterns, the genes encoding those transcripts are said to be differentially spliced. Indeed, differential regulation of mRNA splicing plays crucial roles in biology, such as regulating organism development, specifying tissue identity, and maintaining cell homeostasis. Aberrant regulation of mRNA splicing has also been observed in many cancers. Thus, having the proper tools to detect differentially spliced genes in RNA-seq datasets is crucial to studying these facets of biology with high-throughput RNA-seq data.\\

The ExCluster package allows users to detect differentially spliced genes between two conditions of RNA-seq data, and also allows for the visualization of differential exon expression within significantly differentially spliced genes. ExCluster requires at least two biological replicates per condition, and requires exactly two conditions per analysis. ExCluster has been designed to more rebustly detect differentially spliced genes than do previous established differential splicing tools. ExCluster also more accurately tunes statistics, to reduce the number of false positives when using the typical 0.05 FDR cutoff.

\section{Quick ExCluster analysis pipeline}
This section assumes some familiarity with R and programming in general, and has much less explanation. If you are confused, please read the detailed step-by-step guide for a basic ExCluster analysis. Also, you can refer to the ExCluster manual, which contains details on ExCluster functions and their inputs.\\

To start, we run \R{GFF\_convert} on our GTF file, saving the GFF file, both of which require file paths to be set to variables:\\

\noindent\R{> library(ExCluster)}\\
\R{> GTF\_file <- "/path/to/file.gtf"}\\
\R{> GFF\_file <- "/path/to/file.ExClust.gff}\\
\R{> GFF <- GFF\_convert(GTF.File=GTF\_file, GFF.File=GFF\_file)}\\

We now have our GFF annotation data frame written out, and stored in the 'GFF' variable. We can now count reads per BAM file -- but the described process requires only your BAM files of interest in the bamDir and sub-directories. We also name bam files with sampleNames. Then we run \R{processCounts}:\\

\noindent\R{> bamDir <- "/path/to/bamDir/"}\\
\R{> bamFiles <- list.files(bamDir,full.names=TRUE, recursive=TRUE,}\\
\indent\R{pattern="*.bam")}\\
\R{> sampleNames <- c("hESC\_cond1\_rep1", "hESC\_cond1\_rep2", "hESC\_cond1\_rep3",}\\
\indent\R{"iPSC\_cond2\_rep1", "iPSC\_cond2\_rep2", "iPSC\_cond2\_rep3")}\\
\R{> normCounts <- processCounts(bam.Files=bamFiles, sample.Names=sampleNames,}\\
\indent\R{annot.GFF=GFF, paired.Reads=TRUE)}\\

Note: if you are counting paired-end reads, we use paired.Reads=TRUE. If some or all of your samples are single-end, use paired.Reads=FALSE (default if unset is FALSE). \R{processCounts} counts reads per exon bin from each BAM file and normalizes them for library size differences -- we assign this output to the normCounts variable.\\

Now we can set up to run \R{ExCluster}, which requires condition numbers, and we should give it an outDir and fileName to write files to. We also want to plot our significant results, so we use plot.Results=TRUE. We do all of this as follows:\\

\noindent\R{> condNums <- c(1,1,1,2,2,2)}\\
\R{> outDir <- "/path/to/ExClustResults/"}\\
\R{> fileName <- "hESC\_vs\_iPSC\_ExClust\_Results"}\\
\R{> ExClustResults <- ExCluster(exon.Counts=normCounts, cond.Nums=condNums,}\\
\indent\R{annot.GFF=GFF, out.Dir=outDir, result.Filename=fileName, }\\
\indent\R{combine.Exons=TRUE, plot.Results=TRUE)}\\

This will run for several hours, and will output the results to the ExClustResults variable, as well as write those same results to outDir/hESC\_vs\_iPSC\_ExClust\_Results.txt. Additionally, all significantly differentially expressed genes will have their exon bin log2FC means and variances plotted in a sub-folder of outDir.\\

The only additional note is the combine.Exons=TRUE argument. Some exons are co-expressed always within a given gene, and can therefore be combined into a 'super-exon'. This increases the power of ExCluster and reduces computation time greatly. However, it may miss aberrant splicing events. If you suspect abberant splicing, you can use combine.Exons=FALSE (also FALSE by default if unset), and run the longer mult-hour ExCluster analysis.\\

\noindent This concludes the quick run explanation.

\section{Detailed ExCluster pipeline explanation}
This Vignette assumes that you have successfully installed ExCluster in R, using Bioconductor. If you have not yet installed ExCluster with Bioconductor, please try to run the following:\\

\noindent\R{> chooseCRANmirror()}\\
\R {> install.packages("BiocManager")}\\
\R{> BiocManager::install("ExCluster")}\\

Please note that BiocManager (and ExCluster) will require R 3.5.1 or newer. If you have trouble downloading the BiocManger package, try selecting a different mirror or contact Bioconductor. If only ExCluster has issues, please contact the package maintainer. Please also remember that ExCluster will not install properly on Windows, as it relies on the Rsubread package.\\

\noindent With ExCluster installed, we must now load the R package as follows:\\

\noindent\R{> library(ExCluster)}\\

The next step in the ExCluster pipeline is to take a GTF genome annotation file, and flatten it into a GFF file, containing non-overlapping exon bins. This is done using the function \R{GFF\_convert}. GTF files may be obtained from sources such as Ensembl, GENCODE, RefSeq, and UCSC. We strongly recommend using GENCODE GTF files, as they were used to test and develop ExCluster. Within the Stanford and Perkins labs, we have generally had more successful bioinformatics analyses with GENCODE annotations.\\

Before we run \R{GFF\_convert}, we must specify the GTF file location, and we should also specify a GFF file location to write the output to. Saving GFF files is not necessary, as the GFF annotations can be carried through the pipeline if conducted in a single session. However, saving GFF files is good practice, incase data needs to be re-analyzed later on. Once a GFF file has been created, it may be re-used in future analyses.\\

\noindent We therefore set up variables to run the \R{GFF\_convert} function as follows:\\

\noindent\R{> GTF\_file <- "/path/to/file.gff"}\\
\R{> GFF\_file <- "/path/to/file.ExClust.gff}\\

Experienced programmers and R users will understand what the above placeholder filepaths mean. If you are confused about the above example, I will use a more specific example: My username on MacOS is 'matanner', and my GFF file is called 'gencode.v23.annotation.gff'. The full path to my GTF file is:

/Users/matanner/Documents/gencode.v23.annotation.gtf\\

\noindent Now that our GTF\_file and GFF\_file variables are correctly set, we can run \R{GFF\_convert}:\\

\noindent\R{> GFF <- GFF\_convert(GTF.File=GTF\_file, GFF.File=GFF\_file)}\\

Once this last command is run, it will both assign the GFF annotation data frame to the GFF variable, and write out your GFF data frame to the GFF\_file location. If you do not wish to write out GFF files, simply do not specify the GFF.File argument, which by default does not write out data.\\

We can now move forward and set up our exon bin read counting with the \R{processCounts} function. For this we need BAM files with two different conditions, and at least two biological replicates per condition. If you are new to analyzing RNA-seq data, and you only have raw fastq files, you will need to align your fastq files to obtain BAM files. This can be done with many programs, such as TopHat2, HISAT2, STAR, and BWA, among others. We recommend HISAT2 as an aligner, as ExCluster was tested with HISAT2 output -- although HISAT2 only writes SAM files, which must be converted to BAM files with SAMtools.\\

Assuming you have your RNA-seq data aligned and in BAM file format, it is helpful if the exact BAM files you want to analyze are in a single folder (different sub-folders are okay). The most important point for this example is to have only your bam files of interest in your bamDir and its sub-folders. Assuming this is the case, we specify our BAM file paths into the 'bamFiles' variable, as follows:\\

\noindent\R{> bamDir <- "/path/to/bamDir/"}\\
\R{> bamFiles <- list.files(bamDir, full.names=TRUE, recursive=TRUE,}\\
\indent\R{pattern="*.bam")}\\
\R{> print(bamFiles)}\\

The last command should list your bamFiles if the process worked correctly. You should see exactly your BAM file paths of interest. If you have too many BAM files, or are missing BAM files, you may need to manually specify the paths in your bamFiles array. Please consult section 4 (after this section) for alternate ways to specify the bamFiles variable. If you see no BAM file paths printed out, you need to re-check and ensure you provided the exactly correct path to the folder containing your BAM files (folder and file paths are case sensitive).\\

With your BAMfiles correctly specified, we must now assign sample names to each bam file, in precise order. Sometimes BAM files will be named 'accepted\_hits.bam', such as outputs from TopHat2. For this reason we specify exactly what we want each sample (BAM file) to be called in the exon bin count table. We specify sample names in the sampleNames variable. These names should not contain spaces -- instead of spaces, use underscores \_ . It also helps to name your samples by condition name and number, and replicate number. For example, if we are comparing human embryonic stem cells (hESCs) to induced pluripotent stem cells (iPSCs), and we have 3 replicates per condition, we can name our samples as follows:\\

\noindent\R{> sampleNames <- c("hESC\_cond1\_rep1", "hESC\_cond1\_rep2",}\\
\indent\R{"hESC\_cond1\_rep3", "iPSC\_cond2\_rep1", "iPSC\_cond2\_rep2",}\\
\indent\R{"iPSC\_cond2\_rep3")}\\

This naming convention is helpful, because it clearly indicates both the name of the condition, and which condition is which number (which determines the direction of exon fold changes). It also unambiguously identifies replicates. Again, we must verify that this order of sample names matches the order of the bamFiles array, or we will have improperly named samples and thus an improper analysis.\\

With our setup complete, we can now run the \R{processCounts} function -- let us assume we are counting paired-end read data, so we also specify paired.Reads=TRUE:\\

\noindent\R{> normCounts <- processCounts(bam.Files=bamFiles, }\\
\indent\R{sample.Names=sampleNames, annot.GFF=GFF, paired.Reads=TRUE)}\\

Again, you will notice that we specified the paired.Reads=TRUE argument, which is important when conducting ExCluster analyses on paired-end data. Unfortunately, ExCluster cannot currently accomodate mixed sample types. In other words, if some of your BAM files are paired-end, and some are single-end, we cannot specify different counting types. If some or all of your samples are single-end, you can set the paired.Reads=FALSE argument. This is OK, as processCounts also normalizes library sizes, which should factor out much of the library size differences between the paired-end and single-end data.\\

The result of the above R command should give you a library size normalized exon bin count matrix, which is stored in the normCounts variable. To check this data frame, you should use ncols() and nrows() and have a number of columns equal to your number of BAM files, and roughly 500-600 thousand rows for human data. You should also sum your columns with apply(), and see millions of reads per column. These checks can be checked as follows:\\

\noindent\R{> ncols(normCounts)}\\
\R{> nrows(normCounts)}\\
\R{> apply(normCounts,2,sum)}\\

Assuming your normCounts read count data frame has counted properly, you may now proceed with the main function of this package, ExCluster! As before, we must set up some variables before we run our function. We must assign unique condition identifiers to the condNums variable, which correspond in exact order to your condition 1 and condition 2 samples in normCounts and bamFiles. For example, if your first 3 samples are condition 1, and your last 3 samples are condition 2, you could set up condNums as follows:\\

\noindent\R{> condNums <- c(1,1,1,2,2,2)}\\

If you have alternating samples in your normCounts data frame, you would use the following command:\\

\noindent\R{> condNums <- c(1,2,1,2,1,2)}\\

If you have more than two conditions, please only run \R{ExCluster} with 2 conditions at once. To compare multiple conditions, you will have to run \R{ExCluster} several times. Once you have ensured the correct columns in normCounts have been given the correct condition numbers in condNums, we must specify a directory and file name to write out our \R{ExCluster} results. Writing out results is not absolutely necessary, but it is strongly advised. The \R{ExCluster} function can take hours to run, and therefore failing to save data could result in data loss, and the need to re-run the multi-hour analysis again. We specify an output folder and file name as follows:\\

\noindent\R{> outDir <- "/Users/username/path/to/ExClustResults/"}\\
\R{> fileName <- "hESC\_vs\_iPSC\_ExClust\_Results"}\\

Note, when you specify the fileName variable string, you need not include the file extension. By default, \R{ExCluster} will add '.txt' as a file extension, and write a tab delimited file. We can now run the \R{ExCluster} function as follows, assigning the ExCluster results to the ExClustResults variable:\\

\noindent\R{> ExClustResults <- ExCluster(exon.Counts=normCounts, cond.Nums=condNums,}\\
\indent\R{annot.GFF=GFF, out.Dir=outDir, result.Filename=fileName,}\\
\indent\R{combine.Exons=TRUE, plot.Results=TRUE)}\\

You will notice that we specified two extra parameters: combine.Exons and plot.Results. Both arguments must be set as logical TRUE or FALSE values. Anything else will likely throw an error, and ExCluster may fail. If they are not set when running \R{ExCluster}, both combine.Exons and plot.Resuts default to FALSE.\\

Very briefly, some exons are always co-expressed together in transcripts. These exons can therefore be combined into 'super-exons', based on GFF transcript definitions. This increases the read depth of these exons and reduces their variance, giving more powerful statistical resolution. It also decreases the number of exons bins analyzed per gene, which greatly cuts down on computation time. HOWEVER, this may cause aberrant splicing events to become missed. If you suspect aberrant splicing may occur in your biological paradigm, MAKE SURE combine.Exons=FALSE! However, if you are conducting a standard analysis where no aberrant splicing should occur, combine.Exons=TRUE will result in greater power and faster computation completion.\\

The \R{plotExonlog2FC} function may be called from within \R{ExCluster} to save extra lines of commands to the pipeline -- this is done by setting plot.Results=TRUE. Here, all genes equal to or less than an FDR cutoff will be called significant, and plotted in a sub-folder of the ExCluster results directory (outDir). By default the FDR.cutoff = 0.05, however you may specify as low as FDR.cutoff=0.01 and as high as FDR.cutoff=0.2 as an extra argument when running \R{ExCluster}. You may also run \R{plotExonlog2FC} on its own. See the manual for more details.\\

\noindent This concludes the step-by-step, detailed explanation of the ExCluster pipeline.

\section{Custom bamFiles variable assignment}
Assuming that you cannot have all of your BAM files exclusively within a given folder, you can manually set your bamFiles array by entering the paths to your BAM files one by one. This is done as follows:\\

\noindent\R{> bamFiles <- c("/path/to/cond1\_rep1.bam",}\\
\indent\R{"/path/to/cond1\_rep2.bam",}\\
\indent\R{"/path/to/cond1\_rep3.bam",}\\
\indent\R{"/path/to/cond2\_rep1.bam",}\\
\indent\R{"/path/to/cond2\_rep2.bam",}\\
\indent\R{"/path/to/cond2\_rep3.bam")}\\

\noindent Just as a reminder, if my username is 'matanner' on a Mac OS system, the full path to my first BAM file might be "/Users/matanner/Documents/iPSC\_RNAseq/hESC\_cond1\_rep2.bam\\

\noindent In this way, you must specify the exact filepath of each BAM file, and ensure that file path is valid. Although slow, this method offers greater control over your analysis.

\section{Running ExCluster "toy dataset" examples}
The ExCluster package contains a small dataset for 3 genes, with 4 BAM files split evenly between two conditions. This toy dataset also includes sub-sampled GTF and GFF files, as well as various ExCluster results objects. If you are having trouble executing ExCluster functions, or simply wish to test the functionality of the package, you may use the following examples to test each function:\\

First we test the GFF\_convert function, which takes a GTF file input and flattens it into a GFF3 file. We run the code as follows:\\

\begin{scriptsize}
<<>>=
library(ExCluster)
# load the sub-sampled GTF file path from the ExCluster package
GTF_file <- system.file("extdata","sub_gen.v23.gtf", package = "ExCluster")
# now run GTF\_file without assigning a GFF\_file to write out, assigning the results to the GFF object
GFF <- GFF_convert(GTF.File=GTF_file)
@
\end{scriptsize}

Next we can run the processCounts function on the small BAM file dataset as follows:\\

\begin{scriptsize}
<<>>=
# specify the path to the ExCluster package
ExClust_Path <- system.file(package="ExCluster")
# now find the bam files within that folder
bamFiles <- list.files(ExClust_Path,recursive=TRUE,pattern="*.bam",full.names=TRUE)
# assign sample names (only 2 replicates per condition in this example)
sampleNames <- c("iPSC_cond1_rep1","iPSC_cond1_rep2","iPSC_cond2_rep1","iPSC_cond2_rep2")
# now run processCounts, with paired.Reads=TRUE because we are counting paired-end data
normCounts <- processCounts(bam.Files=bamFiles, sample.Names=sampleNames, annot.GFF=GFF, paired.Reads=TRUE)
@
\end{scriptsize}

The main function of the package, ExCluster, can be tested with the following code:\\

\begin{scriptsize}
<<>>=
# assign condition numbers to your samples (we have 4 samples, 2 replicates per condition)
condNums <- c(1,1,2,2)
# now we run ExCluster, assigning its output to the ExClustResults variable
# we are not writing out the ExClustResults table, nor are we plotting exons
# we also use combine.Exons=FALSE to discover one 'significant' gene for example plot purposes
ExClust_Results <- ExCluster(exon.Counts=normCounts,cond.Nums=condNums,annot.GFF=GFF, combine.Exons=FALSE)
@
\end{scriptsize}

We can test the exon log2FC plotting function plotExonlog2FC as follows:\\

\begin{scriptsize}
<<>>=
# now we must specify a directory to write images to
# here we use tempdir, but you may substitute another folder path if you wish
outDir <- paste(tempdir(),"/Images/",sep="")
# now we can run our exon log2FC plotting function
plotExonlog2FC(results.Data=ExClust_Results, out.Dir=outDir, plot.Type="PNG")
@
\end{scriptsize}

\section{Formatting ExCluster results as a GRanges object}

In addition to the above methods, users may also use GTF files imported by rtracklayer in conjuction with the rtracklayerGTFtoGFF function, to produce GFF flattened files. This function calls the GFF\_convert function from within itself, and must be given the results of an rtracklayer import() function.\\

Users may convert the results ExCluster to a GRanges objects, as said results contain genomic coordinates for each exon bin. This may allow for easier manipulation of these datastructures outside of the standard ExCluster pipeline described in this vignette. Please note that the plotExonlog2FC function does not accept GRanges objects, instead expecting data in the format previously described in this vignette.\\

Converting ExCluster results to GRanges format can be done as follows:\\
\begin{scriptsize}
<<>>=
GRanges.ExClustResults <- GRangesFromExClustResults(results.ExClust=ExClust_Results)
@
\end{scriptsize}

\section{Authors}
R. Matthew Tanner, William L. Stanford, and Theodore J. Perkins\\
CMM Department, Faculty of Medicine, University of Ottawa\\
Ottawa Hospital Research Institute \\
501 Smyth Box 511, Ottawa ON K1H 8L6\\
Canada\\


\end{document}

\name{GFF_convert}
\alias{GFF_convert}
\title{
Flatten GTF files into GFF format
}
\description{
The GFF_convert function takes a GTF file input and flattens in into a GFF file, which contains discrete, non-overlapping exon bin definitions. Transcripts per exon bin are annotated, and normal gene names are added, in addition the Ensembl IDs.
}
\usage{
GFF_convert(annot.GTF, GTF.File, GFF.File)
}
\arguments{
  \item{annot.GTF}{This argument must be passed GTF file annotations, either in the form of an rtracklayer object from the rtracklayer import function, or a data table directly imported into R from a GTF file, using tab delimiters. The function will automatically detect the formatting of the file in these cases.
  }
  \item{GTF.File}{A full file path, including file name & extension, to the GTF file which will be flattened. This parameter is required. An example file path for a GTF file would be: /Users/username/path/to/file.gtf
    }
    \item{GFF.File}{A full file path to which the flattened GFF file can be written. This argument is not required, although it is strongly recommended that GFF files be saved for future use. An example file path for a GFF file would be: /Users/username/path/to/file.gff
    }
}
\examples{
# load the sub-sampled GTF file path from the ExCluster package
GTF_file <- system.file("extdata","sub_gen.v23.gtf", package = "ExCluster")
# now run GTF_file without assigning a GFF_file to write out
# assign this output to the GFF variable
GFF <- GFF_convert(GTF.File=GTF_file)
}
\value{
This function returns a GFF data frame, which contains non-overlapping exon bins, flattened from a GTF file. When the function GFF_convert is run, its results should usually be assigned to a variable name, such as 'GFF'.
}
\references{
Anders, S., Reyes, A., and Huber, W. (2012). Detecting differential usage of exons from RNA-seq data. Genome Research, 22(10):2008-17.
Lawrence M, Huber W, Pag\`es H, Aboyoun P, Carlson M, et al. (2013) Software for Computing and Annotating Genomic Ranges. PLoS Comput Biol 9(8): e1003118.doi:10.1371/journal.pcbi.1003118
}
\author{
R. Matthew Tanner
}
\note{
The GFF_convert function is based on the ideas from the authors of DEXSeq (Anders et al., 2012). Their dexseq_prepare_annotations.py pipeline for flattening GTF files into GFF format was translated into R. Great thanks are owed to Dylan Siriwardena (MSc) for helping write the initial translated R version.
}

\name{ExCluster}
\alias{ExCluster}
\title{
ExCluster is the main function in this package, which clusters exon bins within each gene to determine signficiantly differentially expressed exon clusters.
}
\description{
The ExCluster function takes an input of normalized exon bin read counts per sample, and only accepts data from exactly two conditions at once, requiring two biological replicates per condition. ExCluster also requires GFF annotations to annotate exon bins
}
\usage{
ExCluster(exon.Counts, cond.Nums, annot.GFF, GFF.File, out.Dir,
    result.Filename, combine.Exons=TRUE, plot.Results=FALSE, FDR.cutoff=0.05)
}
\arguments{
  \item{exon.Counts}{exon.Counts must be assigned a data frame of normalized read counts per exon bin, which are attained from running the processCounts function. For example, if the results of processCounts are assigned to the normCounts variable, exon.Counts is assigned as: exon.Counts=normCounts This input is required.
    }
    \item{cond.Nums}{cond.Nums must be assigned an array of sample numbers corresponding, in exact order, to each BAM file counted and passed into the exon.Counts argument. The length of cond.Nums must also match the number of columns in exon.Counts exactly. For example, analyzing data with 3 biological replicates per each of two conditions, in that order, would be denoted as: cond.Nums=c(1,1,1,2,2,2) cond.Nums must be given at least 2 biological replicates per condition, and exactly 2 conditions -- otherwise, the ExCluster package will throw an error describing the problem. This is a required input.
    }
    \item{annot.GFF}{If this argument is specified, it must be given a GFF annotation data frame, which is generated from the GFF_convert function. For example, it can be specified as annot.GFF=GFF, assuming your GFF data frame is assigned to the GFF variable. Either annot.GFF or GFF.File must be specified. If both are specified, annot.GFF will take priority.
    }
    \item{GFF.File}{If this argument is specified, it must be given a full file path to a GFF file, including file name and extension. For example, this may look like: /Users/username/path/to/file.gff This argument is not required, but either annot.GFF or GFF.File must be specified.
    }
    \item{out.Dir}{The out.Dir argument should be assigned a character string specifying a full folder path where results may be written. This should be a specific folder to the data being analyzed, to avoid writing results from different analyses to the same folder.This argument is not required, but it is STRONGLY recommended that you write out the results of the ExCluster analysis, as this portion of the package can take well over 2 hours to complete. An example path for out.Dir would be: /Users/username/Documents/RNA-seq/project_A/
    }
    \item{result.Filename}{result.Filename should be given a character string specifying the specific filename for the ExCluster results table to be written to, within the out.Dir folder. File extension specification is not necessary, as '.txt.' will be added to the result.Filename regardless. This argument is not required, although it may be helpful to name your results by including both condition names, so the specific comparison made is easily identifiable at  a later point in time. By default, result.Filename will be assigned the value "ExClust_Results". An example filename that may be helpful could be: "ExClust_res_GeneA_shRNA_vs_scramble"
    }
    \item{combine.Exons}{combine.Exons should be assigned a logical value of either TRUE or FALSE. This denotes whether exon bins which are always co-expressed in the same transcripts should be combined into 'super-exons'. Doing this can be helpful to increase exon read depth, and it greatly reduces computation time. However, this should only be done in a standard RNA-seq analysis, when no instances of abberant splicing are predicted. If one suspects aberrant splicing in one of your conditions, this argument should not be set. By default combine.Exons=FALSE
    }
    \item{plot.Results}{plot.Results should be assigned a logical value of either TRUE or FALSE. This determines whether or not the ExCluster function should automatically run the plotExonlog2FC function, and plot exon bin log2FCs for each significantly differentially expressed gene. It is generally helpful to run this alongside your analysis, as it saves time. However, your ExCluster results can be saved and read back into R at a later date, from which you can run the plotExonlog2FC function separately. By default plot.Results is set to FALSE.
    }
    \item{FDR.cutoff}{The FDR.cutoff argument should be assigned a value between 0.01 and 0.2. Using FDR cutoffs outside these bounds is not recommended. This number determines which false discovery rate cutoff will be used to discover significant genes by ExCluster. However, this parameter is only used if plot.Results is specified. By default FDR.cutoff=0.05
    }
}
\examples{
# specify the path to the normCounts file in the ExCluster package
countsPath<- system.file("extdata","normCounts.txt",package="ExCluster")
# now read in the normCounts.txt file
normCounts <- read.table(file=countsPath,header=TRUE,row.names=1,
    stringsAsFactors=FALSE)
# now grab the path to the sub-sampled example GFF file
GFF_file <- system.file("extdata","sub_gen.v23.ExClust.gff3",package="ExCluster")
# assign condition numbers to your samples (we have 4 samples, 2 replicates per condition)
condNums <- c(1,1,2,2)
# now we run ExCluster, assigning its output to the ExClustResults variable
# we are not writing out the ExClustResults table, nor are we plotting exons
# we also use combine.Exons=TRUE, since we are conducting a standard analysis
ExClust_Results <- ExCluster(exon.Counts=normCounts,cond.Nums=condNums,
    GFF.File=GFF_file)
}
\value{
This is the main function of the ExCluster package, and returns a data frame of exon bin log2FCs, log2 read variances, exon clustering per gene, p-values, FDRs, and normalized exon counts. The results of the ExCluster function should typically be assigned to a variable, such as 'ExClustResults'.
}
\references{
Charrad M., Ghazzali N., Boiteau V., Niknafs A. (2014). NbClust: An R Package for Determining the Relevant Number of Clusters in a Data Set. Journal of Statistical Software, 61(6), 1-36.
}
\author{
R. Matthew Tanner
}
\note{
The ExCluster packages uses a scatter-distance index function to optimally cut hierarchically clustered exons within each gene. The code for the functions required were adapted in part, or in whole, from the NbClust R package (Charrad et al., 2014). These sections of the code are explicity specified, and the authors of NbClust provide no warranty for the use and fucntionality of said code.
}

\name{GRangesFromExClustResults}
\alias{GRangesFromExClustResults}
\title{
Converts ExCluster results data table to GRanges format.
}
\description{
GRangesFromExClustResults converts the results from the main ExCluster function into GRanges objects using the GRanges package.
}
\usage{
GRangesFromExClustResults(results.ExClust)
}
\arguments{
    \item{results.ExClust}{This argument should be given the table of data that the main ExCluster function produces. This table of data should contain at minimum 12 columns, plus exin bin read count columns if they are present.
    }
}
\examples{
# grab the path to the sub-sampled ExCluster toy dataset results from the ExCluster function
ExClust_Results <- system.file("extdata","sub_ExClust_results.txt",package="ExCluster")
# read this data into R
ExClust_Results <- read.table(file=ExClust_Results, sep="\t",header=TRUE,stringsAsFactors=FALSE)
# run the GRangesFromExClustResults function
GRanges.ExClustResults <- GRangesFromExClustResults(results.ExClust=ExClust_Results)
}
\value{
This function returns a GRanges object with ExCluster main function results, in the form of a table of data.
}
\references{
Lawrence M, Huber W, Pag\`es H, Aboyoun P, Carlson M, et al. (2013) Software for Computing and Annotating Genomic Ranges. PLoS Comput Biol 9(8): e1003118.doi:10.1371/journal.pcbi.1003118
}
\author{
R. Matthew Tanner
}

\name{processCounts}
\alias{processCounts}
\title{
Counts reads per exon bin for each BAM file, using GFF exon annotations -- also normalizes library sizes.
}
\description{
The processCounts function takes an input of BAM files, and counts reads per exon bin from GFF annotations. GFF annotations may be given either in the form of a data frame assigned to annot.GFF, or the path to a GFF file given to GFF.File. processCounts also requires sample names per BAM file, to unambiguously name the read count columns. Reads may be counted as pairs or individually for paired-end reads, by setting the paired.Reads parameter to TRUE or FALSE.
}
\usage{
processCounts(bam.Files,sample.Names,annot.GFF,GFF.File,paired.Reads, stranded.Reads,
                out.File,temp.Dir=NULL,num.Cores=NULL)
}
\arguments{
  \item{bam.Files}{An array of full file paths, specifying BAM files from RNA-seq experiments for which exon bin reads will be counted. Each item in the array must specify a full file path to a unique BAM file -- duplicate BAM files are not allowed. This is a required input.
    }
    \item{sample.Names}{An array of sample names corresponding, in exact order, to each BAM file provided to the bam.Files argument. The length of sample.Names must also match the length of bam.Files exactly. This is a required input.
    }
    \item{annot.GFF}{If this argument is specified, it must be given a GFF annotation data frame, which is generated from the GFF_convert function. For example, it can be specified as annot.GFF=GFF, assuming your GFF data frame is assigned to the GFF variable. Either annot.GFF or GFF.File must be specified. If both are specified, annot.GFF will take priority.
    }
    \item{GFF.File}{If this argument is specified, it must be given a full file path to a GFF file, including file name and extension. For example, this may look like: /Users/username/path/to/file.gff This argument is not required, but either annot.GFF or GFF.File must be specified.
    }
    \item{paired.Reads}{paired.Reads should be assigned a value of either TRUE or FALSE, and functions as a logical operator to determine whether or not reads should be counted as pairs. For example, if you are counting reads from a BAM file with paired-end reads, you should set paired.Reads=TRUE By default paired.Reads is FALSE, and thus need not be set for single-end reads.
    }
    \item{stranded.Reads}{This argument should be given a logical TRUE/FALSE value, determining whether RNA-seq reads are stranded or unstranded. This argument is set to FALSE by default. If your RNA-seq reads are stranded, use stranded.Reads=TRUE. If you are unsure, either assume that reads are unstranded, or consult the sequencing facility from which you obtained your RNA-seq fastq files.
    }
    \item{out.File}{This argument should be given a variable with a string which contains the full file path to which you wish to write your normalized exon bin counts. For example, you could set outFile<-"/path/to/normCounts.txt", and run processCounts with out.File=outFile
    }
    \item{temp.Dir}{This argument should be given a variable with a string which contains the full file path to a temporary directory to which you wish to write featureCounts temporary files. In some cases this can greatly speed up read counting. By default, featureCounts temporary files will be written to R's tempdir() directory.
    }
    \item{num.Cores}{This argument should be given a positive integer value which corresponds to the number of cores you with featureCounts to use, when exon reads are counted from your GFF file. For example, using 4 cores would look like: num.Cores=4  This argument defaults to using 1 core within the function.
    }
}
\examples{
# specify the path to the ExCluster package
ExClust_Path <- system.file(package="ExCluster")
# now find the bam files within that folder
bamFiles <- list.files(ExClust_Path,recursive=TRUE,pattern="*.bam",
    full.names=TRUE)
# now grab the path to the sub-sampled example GFF file
GFF_file <- system.file("extdata","sub_gen.v23.ExClust.gff3",
    package="ExCluster")
# assign sample names (only 2 replicates per condition in this example)
sampleNames <- c("iPSC_cond1_rep1","iPSC_cond1_rep2","iPSC_cond2_rep1",
    "iPSC_cond2_rep2")
# now run processCounts, with paired.Reads=TRUE for paired-end data
normCounts <- processCounts(bam.Files=bamFiles, sample.Names=sampleNames,
    GFF.File=GFF_file, paired.Reads=TRUE)
}
\value{
This function returns normalized exon counts per non-overlapping exon bin in the form of a data frame, in which the user has manually entered sample names which will be present as column headers. This should typically be assigned to a variable when run, such as 'normCounts'.
}
\references{
Liao, Y., Smyth, G.K., and Shi, W. (2013). The Subread aligner: fast, accurate and scalable read
mapping by seed-and-vote. Nucleic Acids Research, 41(10):e108.
}
\author{
R. Matthew Tanner
}
\note{
processCounts runs the featureCounts function from the Rsubread package (Liao et al., 2013). As their function is being called from an external package, the Rsubread authors and license owners make no warranties as to its performance.
}

\name{plotExonlog2FC}
\alias{plotExonlog2FC}
\title{
The plotExonlog2FC function plots exon log2FC means and variances for significantly differentially expressed genes in ExCluster results.
}
\description{
This function takes an input of ExCluster's results dataframe, as well as a directory to write files to, and plots exon bin log2FC means and variances for each significantly differentially expressed gene. Significant genes are determined by applying an FDR cutoff to the ExCluster results. By default, FDR.cutoff=0.05, although this parameter may be set between 0.01 and 0.2.
}
\usage{
plotExonlog2FC(results.Data, out.Dir, FDR.cutoff, plot.Type)
}
\arguments{
  \item{results.Data}{This argument should be assigned a data frame containing the results from an ExCluster analysis, obtained by running the function ExCluster. This argument is required to be set. Assuming ExCluster results are assigned to the variable clustResults, this parameter woud be set as follows: results.Data=clustResults
    }
    \item{out.Dir}{The outDir argument should be assigned a character string specifying a full folder path where exon bin images per gene may be written. To keep results coherent, this should be a sub-folder within the folder the original ExCluster results were written to. This directory need not exist, as R will recursively create it. However, this argument is required. For example, if ExCluster results were written to the folder /Users/username/Documents/RNA-seq/project_A/ then the folder the images should be something like: /Users/username/Documents/RNA-seq/project_A/exon_log2FC_images/
    }
    \item{FDR.cutoff}{The FDR.cutoff argument should be assigned a value between 0.01 and 0.2. Using FDR cutoffs outside these bounds is not recommended. This number determines which false discovery rate cutoff will be used to discover significant genes by ExCluster. This parameter is not required, and by default FDR.cutoff=0.05
    }
    \item{plot.Type}{The plot.Type function should be assigned values of either "bitmap" or "PNG". Both inputs will plot PNG images. Either Ghostscript or X11 forwarding is required for plotting images on various operating systems. Default = "bitmap"
    }
}
\examples{
# grab the file path to sub-sampled results from an ExCluster function analysis
ExClust_respath <- system.file("extdata","sub_ExClust_results.txt",
    package="ExCluster")
# load this data into R
ExClust_Results <- read.table(file=ExClust_respath,sep="\t",header=TRUE,
    row.names=1,stringsAsFactors=FALSE)
# now we must specify a directory to write images to
outDir <- paste(tempdir(),"/Images/",sep="")
# now we can run our exon log2FC plotting function
plotExonlog2FC(results.Data=ExClust_Results, out.Dir=outDir)
}
\value{
This function does not return any values, but instead plots exon log2FC means and variances for significant results from an ExCluster function results data.frame.
}
\author{
R. Matthew Tanner
}
